
#include "StdAfx.h"

#include "2DRenderableElements.h"
#include "Scene.h"

void ColorRectangle::Render(const D3DRenderDevice &device, const Scene &scene)
{
	Color2DVertex vTriangle[4] = {
		Color2DVertex(float(m_X1), float(m_Y1), 1.0f, 1.0f, m_Color),
		Color2DVertex(float(m_X2), float(m_Y1), 1.0f, 1.0f, m_Color),
		Color2DVertex(float(m_X1), float(m_Y2), 1.0f, 1.0f, m_Color),
		Color2DVertex(float(m_X2), float(m_Y2), 1.0f, 1.0f, m_Color)
	};

	D3DXMATRIX *mat = scene.matrixStack->GetTop();

	D3DXVec4Transform(&vTriangle[0].pos, &vTriangle[0].pos, mat);
	D3DXVec4Transform(&vTriangle[1].pos, &vTriangle[1].pos, mat);
	D3DXVec4Transform(&vTriangle[2].pos, &vTriangle[2].pos, mat);
	D3DXVec4Transform(&vTriangle[3].pos, &vTriangle[3].pos, mat);

    device->SetRenderState(D3DRS_CLIPPING			, TRUE);
    device->SetRenderState(D3DRS_LIGHTING			, FALSE);
    device->SetRenderState(D3DRS_FOGENABLE			, FALSE);

	device->SetRenderState(D3DRS_ZENABLE			, FALSE);
	device->SetRenderState(D3DRS_ZWRITEENABLE		, FALSE);
	device->SetRenderState(D3DRS_ZFUNC				, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHATESTENABLE	, FALSE);
	device->SetRenderState(D3DRS_ALPHAREF			, 0);
	device->SetRenderState(D3DRS_ALPHAFUNC			, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHABLENDENABLE	, TRUE);
	device->SetRenderState(D3DRS_SRCBLEND			, D3DBLEND_INVSRCALPHA);
	device->SetRenderState(D3DRS_DESTBLEND			, D3DBLEND_SRCALPHA);

	device->SetTextureStageState(0, D3DTSS_COLOROP		, D3DTOP_SELECTARG2);
	device->SetTextureStageState(0, D3DTSS_COLORARG1	, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_COLORARG2	, D3DTA_CURRENT);
	device->SetTextureStageState(0, D3DTSS_ALPHAOP		, D3DTOP_SELECTARG2);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG1	, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG2	, D3DTA_CURRENT);

	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	device->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

    device->SetVertexShader(D3DFVF_XYZRHW | D3DFVF_DIFFUSE);
    device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, vTriangle, sizeof(*vTriangle));
}

void TextureRectangle::Render(const D3DRenderDevice &device, const Scene &scene)
{
	ColorTexture2DVertex vTriangle[4] = {
		ColorTexture2DVertex(float(m_X1), float(m_Y1), 1.0f, 1.0f, m_Color, 0.F, 0.F),
		ColorTexture2DVertex(float(m_X2), float(m_Y1), 1.0f, 1.0f, m_Color, 1.F, 0.F),
		ColorTexture2DVertex(float(m_X1), float(m_Y2), 1.0f, 1.0f, m_Color, 0.F, 1.F),
		ColorTexture2DVertex(float(m_X2), float(m_Y2), 1.0f, 1.0f, m_Color, 1.F, 1.F)
	};

	D3DXMATRIX *mat = scene.matrixStack->GetTop();

	D3DXVec4Transform(&vTriangle[0].pos, &vTriangle[0].pos, mat);
	D3DXVec4Transform(&vTriangle[1].pos, &vTriangle[1].pos, mat);
	D3DXVec4Transform(&vTriangle[2].pos, &vTriangle[2].pos, mat);
	D3DXVec4Transform(&vTriangle[3].pos, &vTriangle[3].pos, mat);

	if (!m_Texture && !m_TextureName.empty()) {
		m_Texture = device.GetTexture(m_TextureName);
	}

    device->SetRenderState(D3DRS_CLIPPING			, TRUE);
    device->SetRenderState(D3DRS_LIGHTING			, FALSE);
    device->SetRenderState(D3DRS_FOGENABLE			, FALSE);

	device->SetRenderState(D3DRS_ZENABLE			, FALSE);
	device->SetRenderState(D3DRS_ZWRITEENABLE		, FALSE);
	device->SetRenderState(D3DRS_ZFUNC				, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHATESTENABLE	, FALSE);
	device->SetRenderState(D3DRS_ALPHAREF			, 0);
	device->SetRenderState(D3DRS_ALPHAFUNC			, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHABLENDENABLE	, TRUE);
	device->SetRenderState(D3DRS_SRCBLEND			, D3DBLEND_SRCALPHA);
	device->SetRenderState(D3DRS_DESTBLEND			, D3DBLEND_INVSRCALPHA);

	if (!m_Texture) {
		device->SetTextureStageState(0, D3DTSS_COLOROP	, D3DTOP_SELECTARG2);
		device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		device->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
		device->SetTextureStageState(0, D3DTSS_ALPHAOP	, D3DTOP_SELECTARG2);
		device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_CURRENT);
	} else {
		device->SetTexture(0, m_Texture);
		device->SetTextureStageState(0, D3DTSS_COLOROP	, D3DTOP_MODULATE);
		device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		device->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
		device->SetTextureStageState(0, D3DTSS_ALPHAOP	, D3DTOP_MODULATE);
		device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
		device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_CURRENT);
	}
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT00 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT01 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT10 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT11 			, 0);
	device->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX			, 0);
	device->SetTextureStageState(0, D3DTSS_ADDRESSU 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSV 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSW 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_BORDERCOLOR				, 0);
	device->SetTextureStageState(0, D3DTSS_MAGFILTER				, D3DTEXF_POINT);
	device->SetTextureStageState(0, D3DTSS_MINFILTER				, D3DTEXF_POINT);
	device->SetTextureStageState(0, D3DTSS_MIPFILTER				, D3DTEXF_POINT);
	device->SetTextureStageState(0, D3DTSS_MIPMAPLODBIAS			, 0);
	device->SetTextureStageState(0, D3DTSS_MAXMIPLEVEL				, 0);
	device->SetTextureStageState(0, D3DTSS_MAXANISOTROPY			, 1);
	device->SetTextureStageState(0, D3DTSS_BUMPENVLSCALE			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVLOFFSET			, 0);
	device->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS	, D3DTTFF_DISABLE);

	device->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
	device->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);

    device->SetVertexShader(D3DFVF_XYZRHW | D3DFVF_DIFFUSE | D3DFVF_TEX1);
    device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, vTriangle, sizeof(*vTriangle));
}

void Text::Render(const D3DRenderDevice &device, const Scene &scene)
{
	if (!m_Font) {
		m_Font = device.GetFont(m_FontName);
		if (!m_Font) {
			return;
		}
	}

	D3DXMATRIX *mat = scene.matrixStack->GetTop();

	D3DXVECTOR4 pos(m_X, m_Y, 0.F, 1.F);

	D3DXVec4Transform(&pos, &pos, mat);

	RECT rect;
	rect.left = pos.x;
	rect.top = pos.y;
	rect.right = 800 + pos.x;
	rect.bottom = 600 + pos.y;
	if (rect.top > 600 || rect.bottom < 0) {
		return;
	}
	m_Font->Begin();
	m_Font->DrawText(m_Text.c_str(), -1, &rect, DT_LEFT | DT_TOP | DT_NOPREFIX, m_Color);
	m_Font->End();
}

void CenteredText::Render(const D3DRenderDevice &device, const Scene &scene)
{
	if (!m_Font) {
		m_Font = device.GetFont(m_FontName);
		if (!m_Font) {
			return;
		}
	}

	D3DXMATRIX *mat = scene.matrixStack->GetTop();

	D3DXVECTOR4 pos(0.F, m_Y, 0.F, 1.F);

	D3DXVec4Transform(&pos, &pos, mat);

	RECT rect;
	rect.left = 0;
	rect.top = pos.y;
	rect.right = 800;
	rect.bottom = 600 + pos.y;
	if (rect.top > 600 || rect.bottom < 0) {
		return;
	}
	m_Font->Begin();
	m_Font->DrawText(m_Text.c_str(), -1, &rect, DT_LEFT | DT_TOP | DT_NOPREFIX | DT_CALCRECT, m_Color);
	rect.left += (800 - rect.right) / 2 + pos.x;
	rect.right = 800 + rect.left;
	m_Font->DrawText(m_Text.c_str(), -1, &rect, DT_LEFT | DT_TOP | DT_NOPREFIX, m_Color);
	m_Font->End();
}
