
#ifndef _2DRenderableElements_H_INCLUDED
#define _2DRenderableElements_H_INCLUDED

#pragma once

#include "RenderableElement.h"
#include "FontManager.h"

#include <d3d8.h>
#include <d3dx8.h>

struct Color2DVertex {
	D3DXVECTOR4	pos;
	D3DCOLOR  color;

	Color2DVertex() {}
	Color2DVertex(const D3DXVECTOR4 &pos_, const D3DCOLOR &color_): pos(pos_), color(color_) {}
	Color2DVertex(float x, float y, float z, float rhw, const D3DCOLOR &color_): pos(x, y, z, rhw), color(color_) {}
};

struct ColorTexture2DVertex {
	D3DXVECTOR4	pos;
	D3DCOLOR	color;
	D3DXVECTOR2	uv;

	ColorTexture2DVertex() {}
	ColorTexture2DVertex(const D3DXVECTOR4 &pos_, const D3DCOLOR &color_, const D3DXVECTOR2 &uv_): pos(pos_), color(color_), uv(uv_) {}
	ColorTexture2DVertex(float x, float y, float z, float rhw, const D3DCOLOR &color_, float u, float v): pos(x, y, z, rhw), color(color_), uv(u, v) {}
};

struct ColorRectangle: public RenderableElement {

	unsigned int m_X1;
	unsigned int m_Y1;
	unsigned int m_X2;
	unsigned int m_Y2;
	D3DCOLOR m_Color;

	ColorRectangle(unsigned int x, unsigned int y, unsigned int w, unsigned int h, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255)):
		m_X1(x),
		m_Y1(y),
		m_X2(x+w),
		m_Y2(y+h),
		m_Color(color)
		{}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene);
};

struct TextureRectangle: public RenderableElement {

	unsigned int m_X1;
	unsigned int m_Y1;
	unsigned int m_X2;
	unsigned int m_Y2;
	std::string m_TextureName;
	IDirect3DTexture8 *m_Texture;
	D3DCOLOR m_Color;

	TextureRectangle(unsigned int x, unsigned int y, unsigned int w, unsigned int h, const std::string &name, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255)):
		m_X1(x),
		m_Y1(y),
		m_X2(x+w),
		m_Y2(y+h),
		m_TextureName(name),
		m_Texture(NULL),
		m_Color(color)
		{}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene);
};

struct Text: public RenderableElement {

	unsigned int	m_X, m_Y;
	std::string		m_Text;
	D3DCOLOR		m_Color;
	FontName		m_FontName;
	ID3DXFont *		m_Font;

	Text():
		m_X(0),
		m_Y(0),
		m_Text("<No text!!>"),
		m_Color(D3DCOLOR_ARGB(255, 255, 255, 255)),
		m_FontName(GetFontName("Arial", 36, false, false, false)),
		m_Font(0)
	{}

	Text(unsigned int x, unsigned int y, const std::string &text, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255), const FontName &fontname = GetFontName("Arial", 36, false, false, false)):
		m_X(x),
		m_Y(y),
		m_Text(text),
		m_Color(color),
		m_FontName(fontname),
		m_Font(0)
	{}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene);
};

struct CenteredText: public RenderableElement {

	unsigned int	m_Y;
	std::string		m_Text;
	D3DCOLOR		m_Color;
	FontName		m_FontName;
	ID3DXFont *		m_Font;

	CenteredText():
		m_Y(0),
		m_Text("<No text!!>"),
		m_Color(D3DCOLOR_ARGB(255, 255, 255, 255)),
		m_FontName(GetFontName("Arial", 36, false, false, false)),
		m_Font(0)
	{}

	CenteredText(unsigned int y, const std::string &text, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255), const FontName &fontname = GetFontName("Arial", 36, false, false, false)):
		m_Y(y),
		m_Text(text),
		m_Color(color),
		m_FontName(fontname),
		m_Font(0)
	{}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene);
};

struct Renderable2DGroup: public RenderableGroup {

	void AddText(unsigned int x, unsigned int y, const std::string &text, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255), const FontName &fontname = GetFontName("Arial", 36, false, false, false)) {
		Add(new Text(x, y, text, color, fontname));
	}

	void AddCenteredText(unsigned int y, const std::string &text, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255), const FontName &fontname = GetFontName("Arial", 36, false, false, false)) {
		Add(new CenteredText(y, text, color, fontname));
	}

	void AddColorRectangle(unsigned int x, unsigned int y, unsigned int w, unsigned int h, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255)) {
		Add(new ColorRectangle(x, y, w, h, color));
	}

	void AddTextureRectangle(unsigned int x, unsigned int y, unsigned int w, unsigned int h, const std::string &name, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255)) {
		Add(new TextureRectangle(x, y, w, h, name, color));
	}
};

#endif
