
#include "StdAfx.h"

#include "BackgroundGrid.h"
#include "Scene.h"
#include "VolumetricFog.h"

struct MYVERTEX {
	D3DXVECTOR4 pos;
	D3DCOLOR	color;
};

BackgroundGridData::BackgroundGridData():
	GeometryData(D3DFVF_XYZRHW | D3DFVF_DIFFUSE, D3DPT_TRIANGLELIST, true),
	m_GridX(40),
	m_GridY(30)
{
}

BackgroundGridData::~BackgroundGridData()
{
}

void BackgroundGridData::ReloadData(const D3DRenderDevice &device)
{
	m_MaxVertices	= (m_GridX + 1) * (m_GridY + 1);
	m_MaxPrimitives	= m_GridX * m_GridY * 2;
	m_MaxIndices	= m_MaxPrimitives*3;
	CreateBuffers(device);

	WORD *iData;
	if (OpenIndexBuffer(device, iData)) {
		unsigned int x, y;
		for (y = 0; y < m_GridY; ++y) {
			for (x = 0; x < m_GridX; ++x) {
				iData[0] = x   + (y+1) * (m_GridX+1);
				iData[1] = x   + (y  ) * (m_GridX+1);
				iData[2] = x+1 + (y+1) * (m_GridX+1);
				iData[3] = x+1 + (y+1) * (m_GridX+1);
				iData[4] = x   + (y  ) * (m_GridX+1);
				iData[5] = x+1 + (y  ) * (m_GridX+1);
				iData += 6;
			}
		}
		CloseIndexBuffer();
	}
}

void BackgroundGridData::Render(const D3DRenderDevice &device, const Scene &scene)
{
	if (!m_VB) {
		UnloadData();
		ReloadData(device);
		if (!m_VB) {
			return;
		}
	}

	float frFar = scene.zFar * 1.05F;	// Just a little bit extra.
	D3DXVECTOR4 farPos(frFar, frFar, frFar, frFar);

	MYVERTEX *vData;
	if (OpenVertexBuffer(device, vData)) {
		unsigned int x, y;
		for (y = 0; y <= m_GridY; ++y) {
			for (x = 0; x <= m_GridX; ++x) {
				float fx = float(x) * 2.F / float(m_GridX) - 1.F;
				float fy = float(y) * 2.F / float(m_GridY) - 1.F;
				D3DXVECTOR4 pos(fx * frFar, fy * frFar, frFar, frFar);
				D3DXVec4Transform(&pos, &pos, &scene.viewprojMatrixInverse);
				D3DXCOLOR fog = ApplyFogsToVertex(scene.distanceFog, scene.volFogList, scene.numVolFog, D3DXVECTOR3(pos.x, pos.y, pos.z), scene.cameraPos);
				const float foga1 = 1.F - fog.a;
				fog.r *= foga1;
				fog.g *= foga1;
				fog.b *= foga1;
				fog.a = 1.F;
				vData->pos = D3DXVECTOR4((fx + 1.F) * 400.F, (-fy + 1.F) * 300.F, 1.F, 2.F);
				vData->color = fog;
				++vData;
			}
		}
		CloseVertexBuffer();

		GeometryData::Render(device, scene);
	}
}

void BackgroundGrid::Render(const D3DRenderDevice &device, const Scene &scene)
{
	device->SetRenderState(D3DRS_CLIPPING				, FALSE);

	device->SetRenderState(D3DRS_STENCILENABLE			, FALSE);
	device->SetRenderState(D3DRS_STENCILFAIL			, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILZFAIL			, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILPASS			, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILFUNC			, D3DCMP_ALWAYS);
	device->SetRenderState(D3DRS_STENCILREF 			, 0);
	device->SetRenderState(D3DRS_STENCILMASK			, 0xFFFFFFFF);
	device->SetRenderState(D3DRS_STENCILWRITEMASK		, 0xFFFFFFFF);

	device->SetRenderState(D3DRS_FILLMODE				, D3DFILL_SOLID);
	device->SetRenderState(D3DRS_LASTPIXEL				, TRUE);
	device->SetRenderState(D3DRS_SHADEMODE				, D3DSHADE_GOURAUD);
	device->SetRenderState(D3DRS_DITHERENABLE			, FALSE);
	device->SetRenderState(D3DRS_EDGEANTIALIAS			, FALSE);
	device->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS	, FALSE);
	device->SetRenderState(D3DRS_MULTISAMPLEMASK		, 0xFFFFFFFF);
	device->SetRenderState(D3DRS_CLIPPLANEENABLE		, 0);



	device->SetRenderState(D3DRS_ZBIAS					, 0);
	device->SetRenderState(D3DRS_TEXTUREFACTOR			, 0);

	device->SetRenderState(D3DRS_ZENABLE				, TRUE);
	device->SetRenderState(D3DRS_ZWRITEENABLE			, FALSE);
	device->SetRenderState(D3DRS_ZFUNC					, D3DCMP_LESSEQUAL);

	device->SetRenderState(D3DRS_ALPHATESTENABLE		, FALSE);
	device->SetRenderState(D3DRS_ALPHAREF				, 0);
	device->SetRenderState(D3DRS_ALPHAFUNC				, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHABLENDENABLE		, FALSE);
	device->SetRenderState(D3DRS_SRCBLEND				, D3DBLEND_ONE);
	device->SetRenderState(D3DRS_DESTBLEND				, D3DBLEND_ZERO);

	device->SetRenderState(D3DRS_FOGENABLE				, FALSE);
	device->SetRenderState(D3DRS_FOGCOLOR				, 0xFF808000);

	device->SetRenderState(D3DRS_SPECULARENABLE 		, FALSE);

	device->SetTexture(0, NULL);

	device->SetTextureStageState(0, D3DTSS_COLOROP					, D3DTOP_SELECTARG2);
	device->SetTextureStageState(0, D3DTSS_COLORARG1				, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_COLORARG2				, D3DTA_CURRENT);
	device->SetTextureStageState(0, D3DTSS_ALPHAOP					, D3DTOP_SELECTARG2);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG1				, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG2				, D3DTA_CURRENT);

	device->SetTextureStageState(1, D3DTSS_COLOROP					, D3DTOP_DISABLE);
	device->SetTextureStageState(1, D3DTSS_ALPHAOP					, D3DTOP_DISABLE);

	// Vertex pipeline.

	device->SetRenderState(D3DRS_LIGHTING					, FALSE);
	device->SetRenderState(D3DRS_COLORVERTEX				, TRUE);

	device->SetRenderState(D3DRS_VERTEXBLEND				, D3DVBF_DISABLE);
	device->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE	, FALSE);
	device->SetRenderState(D3DRS_TWEENFACTOR				, 0);

	device->SetRenderState(D3DRS_SOFTWAREVERTEXPROCESSING	, FALSE);

	// Triangle pipeline.

	device->SetRenderState(D3DRS_CULLMODE					, D3DCULL_NONE);


	DWORD passes;
	HRESULT hr = device->ValidateDevice(&passes);
	if (FAILED(hr)) {
		char str[4096];
		D3DXGetErrorString(hr, str, sizeof(str));
		OutputDebugString("ValidateDevice() error: ");
		OutputDebugString(str);
		OutputDebugString("\n");
	}

	data.Render(device, scene);
}
