
#include "StdAfx.h"

#include "FileGeometryData.h"
#include "D3DDevice.h"
#include "Scene.h"
#include "VolumetricFog.h"

struct MYVERTEX {
	D3DXVECTOR3	pos;
	D3DXVECTOR3	normal;
	D3DCOLOR	diffuse;
	D3DCOLOR	specular;
	D3DXVECTOR2	uv;
};

struct FileGeometryData::FILEVERTPOS {
	D3DXVECTOR3	pos;
};

struct FileGeometryData::FILEVERTEX {
	D3DXVECTOR3		pos;
	D3DXVECTOR3		normal;
	D3DCOLOR		color;
	D3DXVECTOR2		uv;
	unsigned int	vpos;
};

struct FileGeometryData::FILEFACE {
	unsigned short pidx[3];
	unsigned short idx[3];
	D3DXVECTOR3	pos;
	D3DXVECTOR3	normal;
	float r;
};

enum {
	MF_doubleSided	= 0x01,
	MF_textured		= 0x02,
	MF_colored		= 0x04,
	MF_holes		= 0x08,
};

struct FileGeometryData::FILEMATERIAL {
	unsigned char	flags;
	unsigned char	r, g, b;
	unsigned int	texture;
	unsigned int	numVerts;
	unsigned int	numTris;
};

FileGeometryData::FileGeometryData(const std::string &fname):
	GeometryData(D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE | D3DFVF_SPECULAR | D3DFVF_TEX1, D3DPT_TRIANGLELIST, true),
	m_FName(fname),
	m_NumVertPos(0),
	m_NumVertices(0),
	m_NumPrimitives(0),
	m_NumMaterials(0),
	m_VertPosList(0),
	m_VertexList(0),
	m_FaceList(0),
	m_DrawVertPosWaveList(0),
	m_DrawVertPosFogList(0),
	m_DrawVertPosList(0),
	m_DrawVertexList(0),
	m_DrawFaceList(0),
	m_DrawMatVertexList(0),
	m_DrawMatFaceList(0)
{
}

FileGeometryData::~FileGeometryData() {
	delete[] m_MaterialList;
	delete[] m_VertPosList;
	delete[] m_VertexList;
	delete[] m_FaceList;

	delete[] m_DrawVertPosWaveList;
	delete[] m_DrawVertPosFogList;
	delete[] m_DrawVertPosList;
	delete[] m_DrawVertexList;
	delete[] m_DrawFaceList;
	delete[] m_DrawMatVertexList;
	delete[] m_DrawMatFaceList;
}

void FileGeometryData::ReloadData(const D3DRenderDevice &device)
{
	FILE *f = fopen(("c:\\NewAquaData\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen((m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\..\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\..\\..\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("Media\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\Media\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\..\\Media\\" + m_FName).c_str(), "rb");
	if (!f) f = fopen(("..\\..\\..\\Media\\" + m_FName).c_str(), "rb");
	if (!f) {
		return;
	}

	fread(&m_NumVertPos		, 1, sizeof(m_NumVertPos	), f);
	fread(&m_NumVertices	, 1, sizeof(m_NumVertices	), f);
	fread(&m_NumPrimitives	, 1, sizeof(m_NumPrimitives	), f);
	fread(&m_NumMaterials	, 1, sizeof(m_NumMaterials	), f);

	m_VertPosList	= new FILEVERTPOS	[m_NumVertPos];
	m_VertexList	= new FILEVERTEX	[m_NumVertices];
	m_FaceList		= new FILEFACE		[m_NumPrimitives];
	m_MaterialList	= new FILEMATERIAL	[m_NumMaterials];

	m_DrawVertPosWaveList	= new D3DXVECTOR3	[m_NumVertPos];
	m_DrawVertPosFogList	= new D3DXCOLOR		[m_NumVertPos];
	m_DrawVertPosList		= new bool			[m_NumVertPos];
	m_DrawVertexList		= new unsigned int	[m_NumVertices];
	m_DrawFaceList			= new unsigned int	[m_NumPrimitives];
	m_DrawMatVertexList		= new unsigned int	[m_NumMaterials];
	m_DrawMatFaceList		= new unsigned int	[m_NumMaterials];

	fread(m_VertPosList	, 1, sizeof(*m_VertPosList	) * m_NumVertPos	, f);
	fread(m_VertexList	, 1, sizeof(*m_VertexList	) * m_NumVertices	, f);
	fread(m_FaceList	, 1, sizeof(*m_FaceList		) * m_NumPrimitives	, f);
	fread(m_MaterialList, 1, sizeof(*m_MaterialList	) * m_NumMaterials	, f);

	fclose(f);

	m_MaxVertices	= 0;
	m_MaxPrimitives	= 0;

	unsigned int mat;
	for (mat = 0; mat < m_NumMaterials; ++mat) {
		const FILEMATERIAL &fMat =m_MaterialList[mat];
		if (m_MaxVertices < fMat.numVerts) {
			m_MaxVertices = fMat.numVerts;
		}
		if (m_MaxPrimitives < fMat.numTris) {
			m_MaxPrimitives = fMat.numTris;
		}
	}

	m_MaxVertices	= m_NumVertices;
	m_MaxPrimitives	= m_NumPrimitives;

	m_MaxIndices	= m_MaxPrimitives*3;
}

void FileGeometryData::Render(const D3DRenderDevice &device, const Scene &scene)
{
	if (!m_VertexList) {
		UnloadData();
		ReloadData(device);
		if (!m_VertexList) {
			return;
		}
	}

	D3DXMATRIX *worldMat = scene.matrixStack->GetTop();
	D3DXMATRIX worldMatInverse;
	D3DXMatrixInverse(&worldMatInverse, 0, worldMat);

	D3DXVECTOR3 viewNormal;
	D3DXVec3TransformNormal(&viewNormal, &scene.cameraPos, &worldMatInverse);

	FILEVERTEX *fvData = m_VertexList;
	FILEFACE *	ffData = m_FaceList;

	unsigned int vFace	= 0;
	unsigned int vIdx	= 0;

	memset(m_DrawVertPosList	, 0,	sizeof(*m_DrawVertPosList	) * m_NumVertPos	);
	memset(m_DrawVertexList		, 0xFF, sizeof(*m_DrawVertexList	) * m_NumVertices	);
	memset(m_DrawFaceList		, 0xFF, sizeof(*m_DrawFaceList		) * m_NumPrimitives	);
	memset(m_DrawMatVertexList	, 0xFF, sizeof(*m_DrawMatVertexList	) * m_NumMaterials	);
	memset(m_DrawMatFaceList	, 0xFF, sizeof(*m_DrawMatFaceList	) *	m_NumMaterials	);

	unsigned int *curDrawFace	= m_DrawFaceList;
	unsigned int *curDrawVertex	= m_DrawVertexList;

//	m_DrawVertexList	= new unsigned int	[m_NumVertices];
//	m_DrawMatVertexList	= new unsigned int	[m_NumMaterials];
//	m_DrawMatFaceList	= new unsigned int	[m_NumMaterials];

	unsigned int mat;
	for (mat = 0; mat < m_NumMaterials; ++mat) {
		const FILEMATERIAL &fMat = m_MaterialList[mat];
		unsigned int numVerts = 0;
		unsigned int numFaces = 0;
		unsigned int i;
		for (i = 0; i < fMat.numTris; ++i) {
			if ((fMat.flags & MF_doubleSided) || D3DXVec3Dot(&ffData->normal, &(ffData->pos - viewNormal)) <= 0.F) {
				D3DXVECTOR3 transfVector;
				D3DXVec3TransformCoord(&transfVector, &ffData->pos, worldMat);
				if (transfVector.x - ffData->r <= scene.frustumMaxVals.x
				 && transfVector.y - ffData->r <= scene.frustumMaxVals.y
				 && transfVector.z - ffData->r <= scene.frustumMaxVals.z
				 && transfVector.x + ffData->r >= scene.frustumMinVals.x
				 && transfVector.y + ffData->r >= scene.frustumMinVals.y
				 && transfVector.z + ffData->r >= scene.frustumMinVals.z) {
					*curDrawFace = numFaces;
					++numFaces;
					if (curDrawVertex[ffData->idx[0]] == (unsigned int) -1) {
						curDrawVertex[ffData->idx[0]] = numVerts;
						++numVerts;
					}
					if (curDrawVertex[ffData->idx[1]] == (unsigned int) -1) {
						curDrawVertex[ffData->idx[1]] = numVerts;
						++numVerts;
					}
					if (curDrawVertex[ffData->idx[2]] == (unsigned int) -1) {
						curDrawVertex[ffData->idx[2]] = numVerts;
						++numVerts;
					}
					m_DrawVertPosList[ffData->pidx[0]] = true;
					m_DrawVertPosList[ffData->pidx[1]] = true;
					m_DrawVertPosList[ffData->pidx[2]] = true;
				}
			}
			++ffData;
			++curDrawFace;
		}
		curDrawVertex += fMat.numVerts;
		m_DrawMatVertexList	[mat] = numVerts;
		m_DrawMatFaceList	[mat] = numFaces;
	}

	unsigned int i;
	for (i = 0; i < m_NumVertPos; ++i) {
		if (m_DrawVertPosList[i]) {
			D3DXVECTOR3 &vp = m_DrawVertPosWaveList[i];
			D3DXVec3TransformCoord(&vp, &m_VertPosList[i].pos, worldMat);
			m_DrawVertPosFogList[i] = ApplyFogsToVertex(scene.distanceFog, scene.volFogList, scene.numVolFog, vp, scene.cameraPos);
/*
			float distance = D3DXVec3Length(&(vp - scene.cameraPos)) * .02F;
			vp.x += sin(vp.x + scene.animTime * .1F) * distance;
			vp.y += sin(vp.y + scene.animTime * .1F) * distance;
			vp.z += sin(vp.z + scene.animTime * .1F) * distance;
*/
		}
	}

	device->SetRenderState	(D3DRS_SPECULARENABLE, TRUE);
	device->SetVertexShader	(m_FVF);

	fvData = m_VertexList;
	ffData = m_FaceList;

	curDrawFace		= m_DrawFaceList;
	curDrawVertex	= m_DrawVertexList;

	for (mat = 0; mat < m_NumMaterials; ++mat) {
		const FILEMATERIAL &fMat = m_MaterialList[mat];
		MYVERTEX *	vData = 0;
		WORD *		iData = 0;
		unsigned int numVerts = m_DrawMatVertexList[mat];
		unsigned int numFaces = m_DrawMatFaceList[mat];
		if (numVerts > 0 && numFaces > 0) {
			if (OpenBuffers(device, vData, iData, numVerts, numFaces * 3)) {
				unsigned int firstIndex = m_DynPrimPos;
				unsigned int firstVertex = m_DynVertPos;
				WORD *indices = iData;
				unsigned int i;
				for (i = 0; i < fMat.numTris; ++i) {
					if (*curDrawFace != (unsigned int) -1) {
						const unsigned int i0 = ffData->idx[0];
						const unsigned int i1 = ffData->idx[1];
						const unsigned int i2 = ffData->idx[2];
						indices[0] = curDrawVertex[i0];
						indices[1] = curDrawVertex[i1];
						indices[2] = curDrawVertex[i2];
						indices += 3;
					}
					++ffData;
					++curDrawFace;
				}
				for (i = 0; i < fMat.numVerts; ++i) {
					if (curDrawVertex[i] != (unsigned int) -1) {
						MYVERTEX &vertex = vData[curDrawVertex[i]];
						vertex.pos			= /*fvData->pos +*/ m_DrawVertPosWaveList[fvData->vpos];
						vertex.normal		= fvData->normal;
						vertex.uv			= fvData->uv;

						D3DXCOLOR diffuse	= fvData->color;

						const D3DXCOLOR &fog = m_DrawVertPosFogList[fvData->vpos];
						const float foga1	= 1.F - fog.a;

						vertex.diffuse		= D3DCOLOR_COLORVALUE(fog.a * diffuse.r, fog.a * diffuse.g, fog.a * diffuse.b, diffuse.a);
						vertex.specular		= D3DCOLOR_COLORVALUE(foga1 * fog.r, foga1 * fog.g, foga1 * fog.b, 1.F);
					}

					++fvData;
				}
				CloseBuffers();

				curDrawVertex += fMat.numVerts;

				device->SetStreamSource		(0, m_VB, m_VertexSize);
				device->SetIndices			(m_IB, firstVertex);

				if (m_MaterialList[mat].flags & MF_holes) {
					device->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
					device->SetRenderState(D3DRS_ALPHAREF		, 128);
					device->SetRenderState(D3DRS_ALPHAFUNC		, D3DCMP_GREATER);
				} else {
					device->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
					device->SetRenderState(D3DRS_ALPHAREF		, 0);
					device->SetRenderState(D3DRS_ALPHAFUNC		, D3DCMP_ALWAYS);
				}

				if (m_MaterialList[mat].flags & MF_textured) {
					char fname[256];
					sprintf(fname, "MAP%03d.tga", m_MaterialList[mat].texture);
					device.GetTexture(fname);

					device->SetTexture(0, device.GetTexture(fname));

					device->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
					device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
				} else {
					device->SetTexture(0, NULL);

					device->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG2);
					device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG2);
				}

		/*
				if (m_MaterialList[mat].flags & MF_colored) {
				D3DMATERIAL8 mat;
				mat.Diffuse		= D3DXCOLOR(1.F, 1.F, 1.F, 1.F);
				mat.Ambient		= D3DXCOLOR(1.F, 1.F, 1.F, 1.F);
				mat.Specular	= D3DXCOLOR(0.F, 0.F, 0.F, 1.F);
				mat.Emissive	= D3DXCOLOR(0.F, 0.F, 0.F, 1.F);
				mat.Power		= 0.F;
				device->SetMaterial(&mat);
		*/

				if (m_MaterialList[mat].flags & MF_doubleSided) {
					device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
				} else {
					device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
				}

				device->DrawIndexedPrimitive(m_PrimitiveType, 0, numVerts, firstIndex, numFaces);
			}
		} else {
			curDrawFace += fMat.numTris;
			curDrawVertex += fMat.numVerts;
			ffData += fMat.numTris;
			fvData += fMat.numVerts;
		}
	}


/*
		memset(m_DrawVertexList, 0xFF, sizeof(*m_DrawVertexList) * m_NumVertices);

		unsigned int mat;
		for (mat = 0; mat < m_NumMaterials; ++mat) {
			unsigned int i;
			for (i = 0; i < m_NumPrimitives; ++i) {
//				m_FaceList[i].nx * (m_FaceList[i].x - );
				const FILEFACE &face = m_FaceList[i];
				{
					unsigned int i0 = face.idx[0];
					unsigned int i1 = face.idx[1];
					unsigned int i2 = face.idx[2];
					if (m_DrawVertexList[i0] >= m_NumVertices) {
						i0 = m_DrawVertexList[i0] = vIdx;
						++vIdx;
					} else {
						i0 = m_DrawVertexList[i0];
					}
					if (m_DrawVertexList[i1] >= m_NumVertices) {
						i1 = m_DrawVertexList[i1] = vIdx;
						++vIdx;
					} else {
						i1 = m_DrawVertexList[i1];
					}
					if (m_DrawVertexList[i2] >= m_NumVertices) {
						i2 = m_DrawVertexList[i2] = vIdx;
						++vIdx;
					} else {
						i2 = m_DrawVertexList[i2];
					}
					indices[0] = i0;
					indices[1] = i1;
					indices[2] = i2;
					indices += 3;
					++vFace;
				}
			}
		}

			for (i = 0; i < m_NumPrimitives; ++i) {
//				m_FaceList[i].nx * (m_FaceList[i].x - );
				const FILEFACE &face = m_FaceList[i];
				{
					const unsigned int i0 = face.idx[0];
					const unsigned int i1 = face.idx[1];
					const unsigned int i2 = face.idx[2];
					indices[0] = i0;
					indices[1] = i1;
					indices[2] = i2;
					indices += 3;
					++vFace;
				}
			}

		unsigned int i;
		for (i = 0; i < m_NumVertices; ++i) {
			unsigned int idx = m_DrawVertexList[i];
			const FILEVERTEX &fvertex = m_VertexList[i];
			MYVERTEX &vertex = vData[idx];
			vertex.pos			= fvertex.pos;
			vertex.normal		= fvertex.normal;
			vertex.uv			= fvertex.uv;

			D3DXCOLOR diffuse	= fvertex.color;
//			D3DXCOLOR fog		= ApplyFogsToVertex(&distanceFog, NULL, 0, D3DXVECTOR3(fvertex.x, fvertex.y, fvertex.z), cameraPos);
			D3DXCOLOR fog		= D3DXCOLOR(0.F, 0.F, 0.F, 1.F);
			const float foga1 = 1.F - fog.a;

			vertex.diffuse	= D3DCOLOR_COLORVALUE(fog.a * diffuse.r, fog.a * diffuse.g, fog.a * diffuse.b, fog.a * diffuse.a);
			vertex.specular	= D3DCOLOR_COLORVALUE(foga1 * fog.r, foga1 * fog.g, foga1 * fog.b, 1.F);
		}
		CloseBuffers();
	}
*/
}

void File::Render(const D3DRenderDevice &device, const Scene &scene)
{
//	device->SetTransform(D3DTS_WORLD, scene.matrixStack->GetTop());
	D3DXMATRIX m;
	D3DXMatrixIdentity(&m);
	device->SetTransform(D3DTS_WORLD, &m);

	device->SetRenderState(D3DRS_CLIPPING					, TRUE);

	device->SetRenderState(D3DRS_STENCILENABLE			, FALSE);
	device->SetRenderState(D3DRS_STENCILFAIL				, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILZFAIL				, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILPASS				, D3DSTENCILOP_KEEP);
	device->SetRenderState(D3DRS_STENCILFUNC				, D3DCMP_ALWAYS);
	device->SetRenderState(D3DRS_STENCILREF 				, 0);
	device->SetRenderState(D3DRS_STENCILMASK				, 0xFFFFFFFF);
	device->SetRenderState(D3DRS_STENCILWRITEMASK			, 0xFFFFFFFF);

	device->SetRenderState(D3DRS_FILLMODE					, D3DFILL_SOLID);
	device->SetRenderState(D3DRS_LASTPIXEL				, TRUE);
	device->SetRenderState(D3DRS_SHADEMODE				, D3DSHADE_GOURAUD);
	device->SetRenderState(D3DRS_DITHERENABLE				, FALSE);
	device->SetRenderState(D3DRS_EDGEANTIALIAS			, FALSE);
	device->SetRenderState(D3DRS_MULTISAMPLEANTIALIAS		, FALSE);
	device->SetRenderState(D3DRS_MULTISAMPLEMASK			, 0xFFFFFFFF);
	device->SetRenderState(D3DRS_CLIPPLANEENABLE			, 0);



	device->SetRenderState(D3DRS_ZBIAS					, 0);
	device->SetRenderState(D3DRS_TEXTUREFACTOR			, 0);

	device->SetRenderState(D3DRS_ZENABLE					, TRUE);
	device->SetRenderState(D3DRS_ZWRITEENABLE				, TRUE);
	device->SetRenderState(D3DRS_ZFUNC					, D3DCMP_LESSEQUAL);

	device->SetRenderState(D3DRS_ALPHATESTENABLE			, FALSE);
	device->SetRenderState(D3DRS_ALPHAREF					, 0);
	device->SetRenderState(D3DRS_ALPHAFUNC				, D3DCMP_ALWAYS);

	device->SetRenderState(D3DRS_ALPHABLENDENABLE			, FALSE);
	device->SetRenderState(D3DRS_SRCBLEND					, D3DBLEND_ONE);
	device->SetRenderState(D3DRS_DESTBLEND				, D3DBLEND_ZERO);

	device->SetRenderState(D3DRS_FOGENABLE				, TRUE);
	device->SetRenderState(D3DRS_FOGCOLOR					, 0xFF808000);

	device->SetRenderState(D3DRS_SPECULARENABLE 			, FALSE);

	device->SetTexture(0, NULL);

	device->SetTextureStageState(0, D3DTSS_COLOROP				, D3DTOP_SELECTARG2);
//		device->SetTextureStageState(0, D3DTSS_COLOROP				, D3DTOP_MODULATE);
	device->SetTextureStageState(0, D3DTSS_COLORARG1				, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_COLORARG2				, D3DTA_CURRENT);
	device->SetTextureStageState(0, D3DTSS_ALPHAOP				, D3DTOP_SELECTARG2);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG1				, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG2				, D3DTA_CURRENT);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT00 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT01 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT10 			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVMAT11 			, 0);
	device->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX			, 0);
	device->SetTextureStageState(0, D3DTSS_ADDRESSU 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSV 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_ADDRESSW 				, D3DTADDRESS_WRAP);
	device->SetTextureStageState(0, D3DTSS_BORDERCOLOR			, 0);
	device->SetTextureStageState(0, D3DTSS_MAGFILTER				, D3DTEXF_LINEAR);
	device->SetTextureStageState(0, D3DTSS_MINFILTER				, D3DTEXF_LINEAR);
	device->SetTextureStageState(0, D3DTSS_MIPFILTER				, D3DTEXF_LINEAR);
	device->SetTextureStageState(0, D3DTSS_MIPMAPLODBIAS			, 0);
	device->SetTextureStageState(0, D3DTSS_MAXMIPLEVEL			, 0);
	device->SetTextureStageState(0, D3DTSS_MAXANISOTROPY			, 1);
	device->SetTextureStageState(0, D3DTSS_BUMPENVLSCALE			, 0);
	device->SetTextureStageState(0, D3DTSS_BUMPENVLOFFSET			, 0);
	device->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS	, D3DTTFF_DISABLE);

	device->SetTextureStageState(1, D3DTSS_COLOROP				, D3DTOP_DISABLE);
	device->SetTextureStageState(1, D3DTSS_ALPHAOP				, D3DTOP_DISABLE);

	// Vertex pipeline.

	float fogNear =  50.F;
	float fogFar  = 100.F;

	device->SetRenderState(D3DRS_FOGTABLEMODE				, D3DFOG_NONE);
	device->SetRenderState(D3DRS_FOGVERTEXMODE			, D3DFOG_NONE);
	device->SetRenderState(D3DRS_FOGSTART					, *(DWORD*)&fogNear);
	device->SetRenderState(D3DRS_FOGEND 					, *(DWORD*)&fogFar);
	device->SetRenderState(D3DRS_FOGDENSITY 				, 0);
	device->SetRenderState(D3DRS_RANGEFOGENABLE 			, TRUE);

	device->SetRenderState(D3DRS_LIGHTING					, FALSE);
	device->SetRenderState(D3DRS_AMBIENT					, 0xFF404040);
	device->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE	, D3DMCS_COLOR1);
	device->SetRenderState(D3DRS_SPECULARMATERIALSOURCE 	, D3DMCS_COLOR2);
	device->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE	, D3DMCS_COLOR1);
	device->SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE 	, D3DMCS_MATERIAL);
	device->SetRenderState(D3DRS_LOCALVIEWER				, FALSE);
	device->SetRenderState(D3DRS_COLORVERTEX				, TRUE);
	device->SetRenderState(D3DRS_NORMALIZENORMALS			, TRUE);

	device->SetRenderState(D3DRS_VERTEXBLEND				, D3DVBF_DISABLE);
	device->SetRenderState(D3DRS_INDEXEDVERTEXBLENDENABLE	, FALSE);
	device->SetRenderState(D3DRS_TWEENFACTOR				, 0);

	device->SetRenderState(D3DRS_SOFTWAREVERTEXPROCESSING	, FALSE);

	D3DMATERIAL8 mat;
	mat.Diffuse		= D3DXCOLOR(1.F, 1.F, 1.F, 1.F);
	mat.Ambient		= D3DXCOLOR(1.F, 1.F, 1.F, 1.F);
	mat.Specular	= D3DXCOLOR(0.F, 0.F, 0.F, 1.F);
	mat.Emissive	= D3DXCOLOR(0.F, 0.F, 0.F, 1.F);
	mat.Power		= 0.F;
	device->SetMaterial(&mat);

	// Triangle pipeline.

	device->SetRenderState(D3DRS_CULLMODE					, D3DCULL_CCW);


	DWORD passes;
	HRESULT hr = device->ValidateDevice(&passes);
	if (FAILED(hr)) {
		char str[4096];
		D3DXGetErrorString(hr, str, sizeof(str));
		OutputDebugString("ValidateDevice() error: ");
		OutputDebugString(str);
		OutputDebugString("\n");
	}

	GeometryData *geometry = device.GetGeometry("AquaConv.out");
	if (geometry) {
		geometry->Render(device, scene);
	}
}
