
#include "StdAfx.h"

#include "FontManager.h"
#include "D3DDevice.h"

ID3DXFont *FontManager::GetFont(const D3DRenderDevice &device, const FontName &fontname) const
{
	std::map<FontName, ID3DXFont*>::const_iterator it = m_FontMap.find(fontname);
	if (it == m_FontMap.end()) {
		LOGFONT fontData;
		memset(&fontData, 0, sizeof(fontData));
		fontData.lfHeight		= fontname.size;
		fontData.lfWidth		= fontname.size / 3;
//		fontData.lfCharSet		= ANSI_CHARSET;
		fontData.lfWeight		= fontname.bold? FW_BOLD: FW_NORMAL;
		fontData.lfItalic		= fontname.italic? TRUE: FALSE;
		fontData.lfUnderline	= fontname.underline? TRUE: FALSE;
		strcpy(fontData.lfFaceName, fontname.name.c_str());
		ID3DXFont *font = 0;
		D3DXCreateFontIndirect(device, &fontData, &font);
		if (!font) {
			return NULL;
		}
		m_FontMap[fontname] = font;
		return font;
	} else {
		return it->second;
	}
}

void FontManager::ReleaseFonts()
{
	std::map<FontName, ID3DXFont*>::iterator it;
	for (it = m_FontMap.begin(); it != m_FontMap.end(); ++it) {
		it->second->Release();
	}
	m_FontMap.clear();
}
