
#ifndef FontManager_H_INCLUDED
#define FontManager_H_INCLUDED

#pragma once

#include <d3d8.h>
#include <d3dx8.h>

#include <string>
#include <map>

class D3DRenderDevice;

struct FontName {
	int			size;
	bool		bold;
	bool		italic;
	bool		underline;
	std::string	name;
	bool operator <(const FontName &other) const {
		if (size		< other.size		)	return true;
		if (size		> other.size		)	return false;
		if (bold		< other.bold		)	return true;
		if (bold		> other.bold		)	return false;
		if (italic		< other.italic		)	return true;
		if (italic		> other.italic		)	return false;
		if (underline	< other.underline	)	return true;
		if (underline	> other.underline	)	return false;
		return name < other.name;
	}
};

inline FontName GetFontName(const std::string &name, int size, bool bold, bool italic, bool underline)
{
	FontName fontname;
	fontname.size		= size;
	fontname.bold		= bold;
	fontname.italic		= italic;
	fontname.underline	= underline;
	fontname.name		= name;
	return fontname;
}

class FontManager {

	mutable std::map<FontName, ID3DXFont*> m_FontMap;

public:
	ID3DXFont *GetFont(const D3DRenderDevice &device, const std::string &name, int size, bool bold, bool italic, bool underline) const {
		return GetFont(device, GetFontName(name, size, bold, italic, underline));
	}
	ID3DXFont *GetFont(const D3DRenderDevice &device, const FontName &fontname) const;
	void ReleaseFonts();

	~FontManager() {
		ReleaseFonts();
	}
};

#endif
