// GDCLecture.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

#pragma comment(lib, "d3d8")
#pragma comment(lib, "d3dx8")

#pragma warning(disable: 4786)	// 'identifier' : identifier was truncated to 'number' characters in the debug information

////////////////////////////////////////////////////////////////////////////
////

#include <windows.h>
#include "resource.h"

#include "D3DDevice.h"
#include "VolumetricFog.h"
#include "Scene.h"
#include "GeometryData.h"
#include "2DRenderableElements.h"
#include "FileGeometryData.h"
#include "BackgroundGrid.h"

#include <vector>



D3DXMATRIX D3DXMATRIXIDENTITY(1.F, 0.F, 0.F, 0.F,
							  0.F, 1.F, 0.F, 0.F,
							  0.F, 0.F, 1.F, 0.F,
							  0.F, 0.F, 0.F, 1.F);






class RedirectElement: public RenderableElement {

	RenderableElement *m_Element;

public:
	RedirectElement(RenderableElement *element = 0): m_Element(element) {}

	void SetElement(RenderableElement *element = 0) {
		m_Element = element;
	}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene) {
		if (m_Element) {
			m_Element->Render(device, scene);
		}
	}

};

class TransformElement: public RedirectElement {

	D3DXMATRIX m_Transf;

public:
	TransformElement(const D3DXMATRIX &transf = D3DXMATRIXIDENTITY, RenderableElement *element = 0): RedirectElement(element), m_Transf(transf) {}

	void SetTransform(const D3DXMATRIX &transf) {
		m_Transf = transf;
	}

	void SetTransform(const D3DXMATRIX *transf) {
		m_Transf = *transf;
	}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene) {
		scene.matrixStack->Push();
		scene.matrixStack->MultMatrix(&m_Transf);
		RedirectElement::Render(device, scene);
		scene.matrixStack->Pop();
	}

};




FontName TextFont = GetFontName("Arial", 36, false, false, false);
FontName CodeFont = GetFontName("Courier New", 36, true, false, false);
FontName SmallCodeFont = GetFontName("Courier New", 25, true, false, false);


enum {
	TitleColor = D3DCOLOR_ARGB(255, 255, 128, 128),
	TextColor = D3DCOLOR_ARGB(255, 255, 255, 255),
};

Renderable2DGroup *pageList[256];
int pageHeightList[256];
unsigned int numPages = 0;

#define BEGINPAGE(n) static struct n: public Renderable2DGroup { n() { \
	pageList[numPages] = this; \
	int &y = pageHeightList[numPages] = 0;

#define ENDPAGE(n) \
	y -= 500; \
	if (y < 0) y = 0; \
	y = -y; \
	numPages++; \
	} } n





BEGINPAGE(TextPageTitle)
	AddCenteredText(y +=  50, "Implementing multi-colored volumetric fog", TextColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y +=  50, "without using up texture stages", TextColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 200, "Speaker: Juan Carlos Arvalo-Baeza", TextColor, TextFont);
	AddCenteredText(y += 150, "email: jcab@roningames.com", TextColor, TextFont);
	AddCenteredText(y +=  50, "http:/www.metro.net/jcab/Rumblings/html/index.html", TextColor, TextFont);
ENDPAGE(TextPageTitle);

BEGINPAGE(TextPageContents)
	AddCenteredText(y +=  50, "Topics of discussion", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddText(20, y += 150, "- Getting more colors out of your fog", TextColor, TextFont);
	AddText(20, y +=  40, "- Mathematics: fog formulas", TextColor, TextFont);
	AddText(20, y +=  40, "- Mathematics: volume formulas", TextColor, TextFont);
	AddText(20, y +=  40, "- Volumetric fog ray-casting", TextColor, TextFont);
	AddText(20, y +=  40, "- Implementation details", TextColor, TextFont);
	AddText(20, y +=  40, "- Questions, questions...", TextColor, TextFont);
ENDPAGE(TextPageContents);





BEGINPAGE(TextPageFogColor1)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The problem:", TextColor, TextFont);
	AddText(20, y += 100, "- We don't want to be limited by the hardware", TextColor, TextFont);
	AddText(20, y +=  40, "- Problem: we are limited by typical hardware", TextColor, TextFont);
	AddText(20, y +=  40, "- Can only do one flat fog color per triangle", TextColor, TextFont);
	AddText(20, y +=  40, "- Solution: Emulate what we want using", TextColor, TextFont);
	AddText(100, y +=  40, "the other interpolated colors", TextColor, TextFont);
ENDPAGE(TextPageFogColor1);

BEGINPAGE(TextPageFogColor2)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Typical hardware:", TextColor, TextFont);
	AddText(20, y += 100, "- Interpolates: Diffuse color, Specular color and FogValue", TextColor, TextFont);
	AddText(20, y +=  40, "- color1 = texture * diffuse + specular", TextColor, CodeFont);
	AddText(20, y +=  40, "- color = color1 * fogvalue + fogcolor * (1-fogvalue)", TextColor, CodeFont);
	AddText(20, y +=  40, "- But we want it to interpolate fogcolor too", TextColor, TextFont);
ENDPAGE(TextPageFogColor2);

BEGINPAGE(TextPageFogColor3)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The solution:", TextColor, TextFont);
	AddText(20, y += 100, "- Reorganize the elements from the formulas:", TextColor, TextFont);
	AddText(20, y +=  40, "- color1 = (texture * diffuse + specular) * fogvalue", TextColor, CodeFont);
	AddText(20, y +=  40, "- color2 = fogcolor * (1-fogvalue)", TextColor, CodeFont);
	AddText(20, y +=  40, "- color = color1 + color2", TextColor, CodeFont);
	AddText(20, y +=  40, "- And again:", TextColor, TextFont);
	AddText(20, y +=  40, "- color1 = texture * diffuse*fogvalue", TextColor, CodeFont);
	AddText(20, y +=  40, "- color2 = specular * fogvalue + fogcolor * (1-fogvalue)", TextColor, CodeFont);
	AddText(20, y +=  40, "- color = color1 + color2", TextColor, CodeFont);
ENDPAGE(TextPageFogColor3);

BEGINPAGE(TextPageFogColor4)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The solution:", TextColor, TextFont);
	AddText(20, y += 100, "- Forget about hardware fog", TextColor, TextFont);
	AddText(20, y +=  40, "- Instead, recalculate, per vertex, new colors", TextColor, TextFont);
	AddText(20, y +=  40, "- newdiffuse = diffuse * fogvalue", TextColor, CodeFont);
	AddText(20, y +=  40, "- newspecular = specular * fogvalue + fogcolor * (1-fogvalue)", TextColor, CodeFont);
	AddText(20, y +=  40, "- color = texture * newdiffuse + newspecular", TextColor, CodeFont);
ENDPAGE(TextPageFogColor4);

BEGINPAGE(TextPageFogColor5)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Caveats:", TextColor, TextFont);
	AddText(20, y += 100, "- Different interpolation path: different rendered values", TextColor, TextFont);
	AddText(20, y +=  40, "- Additional per-vertex computations", TextColor, TextFont);
	AddText(20, y +=  40, "- But: values at vertices are correct", TextColor, TextFont);
	AddText(20, y +=  40, "- So: errors end up being small", TextColor, TextFont);
ENDPAGE(TextPageFogColor5);

BEGINPAGE(TextPageFogColor6)
	AddCenteredText(y +=  50, "Getting more colors out of your fog", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Demos and questions", TextColor, TextFont);
ENDPAGE(TextPageFogColor6);




BEGINPAGE(TextPageFogMath1)
	AddCenteredText(y +=  50, "Fog, and how it is mathematically computed", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The classic formulas:", TextColor, TextFont);
	AddText(20, y += 100, "- Typically, fog colors are provided by the program", TextColor, TextFont);
	AddText(20, y +=  40, "- Typically, fog values are calculated from:", TextColor, TextFont);
	AddText(20, y +=  40, "- Values: near distance, far distance", TextColor, TextFont);
	AddText(20, y +=  40, "- dist = distance(vertexpos, camerapos)", TextColor, CodeFont);
	AddText(20, y +=  40, "- fogval = clamp01((dist-near)/(far-near))", TextColor, CodeFont);
	AddText(20, y +=  40, "- That's good for distance (atmospheric) fog", TextColor, TextFont);
ENDPAGE(TextPageFogMath1);

BEGINPAGE(TextPageFogMath2)
	AddCenteredText(y +=  50, "Fog, and how it is mathematically computed", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The volumetric formulas:", TextColor, TextFont);
	AddText(20, y += 100, "- The distance that the light travels through the fog", TextColor, TextFont);
	AddText(20, y +=  40, "- We'll call it fvdist", TextColor, TextFont);
	AddText(20, y +=  40, "- For atmospheric fog, it would be:", TextColor, TextFont);
	AddText(20, y +=  40, "- fvdist = max(dist-near, 0)", TextColor, CodeFont);
	AddText(20, y +=  40, "- (we do the clamping there)", TextColor, TextFont);
ENDPAGE(TextPageFogMath2);

BEGINPAGE(TextPageFogMath3)
	AddCenteredText(y +=  50, "Fog, and how it is mathematically computed", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The new fog formulas:", TextColor, TextFont);
	AddText(20, y += 100, "- Values needed: fvdist, fog density", TextColor, TextFont);
	AddText(20, y +=  40, "- fogval = max(1 - fvdist*density, 0)", TextColor, CodeFont);
	AddText(20, y +=  40, "- For atmospheric fog, the density would be:", TextColor, TextFont);
	AddText(20, y +=  40, "- density = 1/(far - near)", TextColor, CodeFont);
ENDPAGE(TextPageFogMath3);

BEGINPAGE(TextPageFogMath4)
	AddCenteredText(y +=  50, "Fog, and how it is mathematically computed", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Applying more than one fog range:", TextColor, TextFont);
	AddText(20, y += 100, "- We will composite the different fog colors and values:", TextColor, TextFont);
	AddText(20, y +=  40, "- First (further) fog range: fogval1, fogcolor1", TextColor, TextFont);
	AddText(20, y +=  40, "- Second (closer) fog range: fogval2, fogcolor2", TextColor, TextFont);
	AddText(20, y +=  40, "- fogval = clamp01(1 - (1-fogval1) + (1-fogval2))", TextColor, CodeFont);
	AddText(20, y +=  40, "- fogcolor = fogcolor1 * fogval2 + fogcolor2 * (1-fogval2)", TextColor, CodeFont);
ENDPAGE(TextPageFogMath4);




BEGINPAGE(TextPageFogVol1)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "What we neeed:", TextColor, TextFont);
	AddText(20, y += 100, "- Rays will be cast from the camera to every visible vertex", TextColor, TextFont);
	AddText(20, y +=  40, "- We will find the points where they intersect the volumes", TextColor, TextFont);
	AddText(20, y +=  40, "- Volumes we will use: half-space, sphere and ellipsoid", TextColor, TextFont);
	AddText(20, y +=  40, "- Other volumes are also possible (cylinders, convex hulls...)", TextColor, TextFont);
	AddText(20, y +=  40, "- We will use parametric formulas. Parameter is 't'", TextColor, TextFont);
ENDPAGE(TextPageFogVol1);

BEGINPAGE(TextPageFogVol2)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The ray formula:", TextColor, TextFont);
	AddText(20, y += 100, "- ray: X = campos + dist * t", TextColor, CodeFont);
	AddText(20, y +=  40, "- dist = vertexpos - campos", TextColor, CodeFont);
	AddText(20, y +=  40, "- At the camera position, 't' is 0", TextColor, TextFont);
	AddText(20, y +=  40, "- At the vertex position, 't' is 1", TextColor, TextFont);
	AddText(20, y +=  40, "- We will need the values of 't' at the intersections", TextColor, TextFont);
	AddText(20, y +=  40, "- Convex volumes, so we get two values: 't1' and 't2'", TextColor, TextFont);
	AddText(20, y +=  40, "- Can use non-convex volumes, but complicates the algorithm", TextColor, TextFont);
ENDPAGE(TextPageFogVol2);

BEGINPAGE(TextPageFogVol3)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The classic distance fog:", TextColor, TextFont);
	AddText(20, y += 100, "- Values: near and far", TextColor, TextFont);
	AddText(20, y +=  40, "- t1 = near / magnitude(dist)", TextColor, CodeFont);
	AddText(20, y +=  40, "- t2 = far  / magnitude(dist)", TextColor, CodeFont);
ENDPAGE(TextPageFogVol3);

BEGINPAGE(TextPageFogVol4)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The half-space:", TextColor, TextFont);
	AddText(20, y += 100, "- Defined as a point (P) and a normal (N)", TextColor, TextFont);
	AddText(20, y +=  40, "- (X - P) * N > 0", TextColor, CodeFont);
	AddText(20, y +=  40, "- Substituting in the ray formula:", TextColor, TextFont);
	AddText(20, y +=  40, "- (campos + dist * t - P) * N > 0", TextColor, CodeFont);
	AddText(20, y +=  40, "- Rearranging:", TextColor, TextFont);
	AddText(20, y +=  40, "- t > ((P-campos)N) / (distN) (if distN > 0)", TextColor, CodeFont);
	AddText(20, y +=  40, "- t < ((P-campos)N) / (distN) (if distN < 0)", TextColor, CodeFont);
ENDPAGE(TextPageFogVol4);

BEGINPAGE(TextPageFogVol5)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The sphere:", TextColor, TextFont);
	AddText(20, y += 100, "- Defined as a point (P) and a radius (R)", TextColor, TextFont);
	AddText(20, y +=  40, "- (X - P)^2 < R^2", TextColor, CodeFont);
	AddText(20, y +=  40, "- Substituting in the ray formula:", TextColor, TextFont);
	AddText(20, y +=  40, "- (campos + dist * t - P)^2 < R^2", TextColor, CodeFont);
	AddText(20, y +=  40, "- Rearranging:", TextColor, TextFont);
	AddText(20, y +=  40, "- radix = [(campos-P)dist]^2 - dist^2*[(campos-P)^2 - R^2]", TextColor, CodeFont);
	AddText(20, y +=  40, "- t1 = {-(campos-P)dist - sqrt(radix)} / dist^2", TextColor, CodeFont);
	AddText(20, y +=  40, "- t2 = {-(campos-P)dist + sqrt(radix)} / dist^2", TextColor, CodeFont);
ENDPAGE(TextPageFogVol5);

BEGINPAGE(TextPageFogVol6)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The ellipsoid:", TextColor, TextFont);
	AddText(20, y += 100, "- Defined as a point (P) and three sizes (R1, R2, R3)", TextColor, TextFont);
	AddText(20, y +=  40, "- That was an axis-aligned ellipsoid", TextColor, TextFont);
	AddText(20, y +=  40, "- But: in general, can be seen as a transformed unit sphere", TextColor, TextFont);
	AddText(20, y +=  40, "- That's a linear transform: rotation, translation, shear...", TextColor, TextFont);
	AddText(20, y +=  40, "- In general, a 3x4 matrix", TextColor, TextFont);
ENDPAGE(TextPageFogVol6);

BEGINPAGE(TextPageFogVol7)
	AddCenteredText(y +=  50, "The fog volumes", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The ellipsoid:", TextColor, TextFont);
	AddText(20, y += 100, "- So, find the inverse of that transform, and use it on", TextColor, TextFont);
	AddText(40, y +=  40, "the camera position and the vertex position vectors", TextColor, TextFont);
	AddText(20, y +=  40, "- So, apply the sphere formula taking P = 0 and R = 1", TextColor, TextFont);
	AddText(20, y +=  40, "- radix = [camposdist]^2 - dist^2*[campos^2 - 1]", TextColor, CodeFont);
	AddText(20, y +=  40, "- t1 = {-camposdist - sqrt(radix)} / dist^2", TextColor, CodeFont);
	AddText(20, y +=  40, "- t2 = {-camposdist + sqrt(radix)} / dist^2", TextColor, CodeFont);
ENDPAGE(TextPageFogVol7);





BEGINPAGE(TextPageFogImpl1)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "The algorithm:", TextColor, TextFont);
	AddText(20, y += 100, "- Somehow, partition the ray into segments with fog", TextColor, TextFont);
	AddText(20, y +=  40, "- Sort the segments", TextColor, TextFont);
	AddText(20, y +=  40, "- And apply them one after the other, back to front", TextColor, TextFont);
	AddText(20, y +=  40, "- Each segment has fogvalue and fogcolor", TextColor, TextFont);
	AddText(20, y +=  40, "- If two or more volumes share a segment,", TextColor, TextFont);
	AddText(40, y +=  40, "average the color, weighted on the fog values", TextColor, TextFont);
ENDPAGE(TextPageFogImpl1);

BEGINPAGE(TextPageFogImpl2)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Partitioning:", TextColor, TextFont);
	AddText(20, y += 100, "- I do it by defining an accumulation buffer", TextColor, TextFont);
	AddText(20, y +=  40, "- (dcolor, density) = (volcolor * voldensity, voldensity)", TextColor, CodeFont);
	AddText(20, y +=  40, "- And fill buffer with positive and negative entries", TextColor, TextFont);
	AddText(20, y +=  40, "- (for the entry point and the exit point: 't1' and 't2')", TextColor, TextFont);
ENDPAGE(TextPageFogImpl2);

BEGINPAGE(TextPageFogImpl3)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Loop:", TextColor, TextFont);
	AddText(20, y += 100, "- Sort that buffer, and for each entry, apply:", TextColor, TextFont);
	AddText(20, y +=  40, "- accumdcolor += dcolor", TextColor, CodeFont);
	AddText(20, y +=  40, "- accumdensity += density", TextColor, CodeFont);
	AddText(20, y +=  40, "- segmentfogcolor = accumdcolor / accumdensity", TextColor, CodeFont);
	AddText(20, y +=  40, "- segmentfogvalue = 1 - distance * accumdensity", TextColor, CodeFont);
	AddText(20, y +=  40, "- Apply each segment in turn", TextColor, TextFont);
ENDPAGE(TextPageFogImpl3);

BEGINPAGE(TextPageFogImpl4)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Optimizations:", TextColor, TextFont);
	AddText(20, y += 100, "- Do front to back: allows for early exits", TextColor, TextFont);
	AddText(20, y +=  40, "- Special-case when t=0 and t=1", TextColor, TextFont);
	AddText(20, y +=  40, "- When t=0, just use the value to prime the accummulator", TextColor, TextFont);
	AddText(20, y +=  40, "- When t=1, just ignore it", TextColor, TextFont);
	AddText(20, y +=  40, "- Hard-coded sorts for limited number of volumes", TextColor, TextFont);
ENDPAGE(TextPageFogImpl4);

BEGINPAGE(TextPageFogImpl5)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Using vertex buffers:", TextColor, TextFont);
	AddText(20, y += 100, "- Problem: no loops", TextColor, TextFont);
	AddText(20, y +=  40, "- Problem: no conditionals", TextColor, TextFont);
	AddText(20, y +=  40, "- In general: no jumps", TextColor, TextFont);
	AddText(20, y +=  40, "- No problem if it's only atmospheric fog", TextColor, TextFont);
	AddText(20, y +=  40, "- Can do neat effects with it", TextColor, TextFont);
ENDPAGE(TextPageFogImpl5);

BEGINPAGE(TextPageFogImpl6)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Using vertex buffers:", TextColor, TextFont);
	AddText(20, y += 100, "- Ok, ok... let's try", TextColor, TextFont);
	AddText(20, y +=  40, "- The really hard part is the sorting", TextColor, TextFont);
	AddText(20, y +=  40, "- Everything else can be just hard-coded", TextColor, TextFont);
	AddText(20, y +=  40, "- (quite like applying lights, actually)", TextColor, TextFont);
	AddText(20, y +=  40, "- So we'll assume a single volume", TextColor, TextFont);
ENDPAGE(TextPageFogImpl6);

BEGINPAGE(TextPageFogImpl7)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Vertex buffer:", TextColor, TextFont);
	AddText(20, y += 100, "- 37 instructions total", TextColor, TextFont);
	AddText(20, y +=  40, "; r5        = camera pos in world coordinates", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r6.xyz    = camera pos in fog volume coordinates", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r7        = vertex direction in world coordinates", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r8.xyz    = vertex direction in fog volume coordinates", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mov r5, c27", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "m4x3 r6, r5, c50", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "add r7, -r5, r11", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "m4x3 r8, r7, c50", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; calculate t1 and t2 for the sphere", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "dp3 r9.x, r6, r8", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "dp3 r9.y, r6, r6", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "dp3 r9.z, r8, r8", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r9.y, r9.y, r9.z, -r9.z", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r9.y, r9.x, r9.x, -r9.y", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "max r9.y, r9.y, c0.x    ; clamp to zero before square root.", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "rsq r9.w, r9.y          ; if negative, then no fog will be used -> good", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r9.y, r9.y, r9.w    ; sqr(x) = x*rsq(x)", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "rcp r9.w, r9.z", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r9.x, -r9.x, r9.w", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r9.z, r9.w, -r9.y, r9.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r9.w, r9.w,  r9.y, r9.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "max r9.zw, r9, c0.x     ;", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "min r9.zw, r9, c1.x     ; clamp between 0 and 1", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "add r9.y, -r9.z, r9.w", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; here:", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r9.y = t2 - t1", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r9.z = t1", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r9.w = t2", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5.xyz, r9.y, r7", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "dp3 r5.w, r5, r5", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "rsq r5.x, r5.w", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5.x, r5.x, r5.w", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5.x, c53.w, r5.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "min r5.x, r5.x, c1.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "; r5.x is the fogval for the volume", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5.x, r5.x, r5.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r6.xyz, c53, r5.x   ; blend the two fog colors", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "add r5.x, -r5.x, c1.x", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r4.xyz, r4, r5.x, r6", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5.y, r5.x, r2.x    ; r5.y = the global fog value", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "add r5.z, -r5.y, c1.x   ; r5.z = 1-fog", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul oD0, r1, r5.y    ; dim down by fog", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul oD1, r4, r5.z ; fogcolor", TextColor, SmallCodeFont);
ENDPAGE(TextPageFogImpl7);

BEGINPAGE(TextPageFogImpl8)
	AddCenteredText(y +=  50, "Implementation", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Vertex buffer:", TextColor, TextFont);
	AddText(20, y += 100, "- We can even try the sorting:", TextColor, TextFont);
	AddText(20, y +=  40, "- We need to be able to swap two values or not", TextColor, TextFont);
	AddText(40, y +=  40, "depending on another two values", TextColor, TextFont);
	AddText(20, y +=  40, "- We will use r0.x and r0.y as the keys", TextColor, TextFont);
	AddText(20, y +=  40, "- We will use r1 and r2 as the values", TextColor, TextFont);
	AddText(20, y +=  40, "- We will use r3 and r4 as temporaries", TextColor, TextFont);
	AddText(20, y +=  40, "- We will use r5 and r6 as the destination", TextColor, TextFont);
	AddText(20, y +=  40, "slt r3, r0.x, r0.y", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "add r4, one, -r3", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r5, r3, r1", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r5, r4, r2, r5", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mul r6, r4, r1", TextColor, SmallCodeFont);
	AddText(20, y +=  20, "mad r6, r3, r2, r6", TextColor, SmallCodeFont);
	AddText(20, y +=  40, "- Result: r5 = r0.x < r0.y? r1: r2", TextColor, TextFont);
	AddText(20, y +=  40, "- Result: r6 = r0.x < r0.y? r2: r1", TextColor, TextFont);
ENDPAGE(TextPageFogImpl8);




BEGINPAGE(TextPageFogEnd)
	AddCenteredText(y +=  50, "Conclusion", TitleColor, GetFontName("Arial", 48, false, false, false));
	AddCenteredText(y += 100, "Demos and questions", TextColor, TextFont);
ENDPAGE(TextPageFogEnd);



/*
BEGINPAGE(TextPage1)
	AddText				(100, y += 50, "Hello World!!", TextColor, TextFont);
	AddCenteredText		(y += 50, "Hello World Again!!", D3DCOLOR_ARGB(255, 255, 0, 255), CodeFont);
	AddTextureRectangle	(100, 400, 200, 200, "forest.dds");
	AddColorRectangle	(300, 400, 200, 200, D3DCOLOR_ARGB(128, 255, 0, 32));
	y = 600;
ENDPAGE(TextPage1);
*/


struct Background: public TextureRectangle {

	Background(const D3DCOLOR &color = D3DCOLOR_ARGB(224, 0, 0, 32)): TextureRectangle(0, 0, 800, 600, "", color) {}
	Background(const std::string &name, const D3DCOLOR &color = D3DCOLOR_ARGB(255, 255, 255, 255)): TextureRectangle(0, 0, 800, 600, name, color) {}

};




const float fogFar = 20.F;

class GDCLectureScene: public Scene {
	RenderableGroup theScene;
	RenderableGroup the3DScene;
	RenderableGroup the2DScene;

	TransformElement	transform2D;
	Background			theBackground;
	TransformElement	pageRedir;

	FogVolumeDistanceFlat		flatWaterFog;
	FogVolumeDistanceVGradBlam	niceWaterFog;
	FogVolumeDistanceHGrad		demoWaterFog;

	FogEllipsoid			demoEllipsoid;
	FogEllipsoid			demoEllipsoid2;
	FogEllipsoid			caveDarkness;
	FogEllipsoid			cloudFog;

	FogVolume *				volumeList[2];

	unsigned int			curPage;

	BackgroundGrid	background3D;
	File			terrain;

	DWORD lastRenderTime;

	bool	panOut2D;
	DWORD	panOut2DStartTime;
	bool	panIn2D;
	DWORD	panIn2DStartTime;
	bool	visible2D;

	DWORD lastCursorX;
	DWORD lastCursorY;

	int scroll2D;

	enum MouseState {
		MS_Scroll,
		MS_3DCamera,
	} MouseState;

public:
	GDCLectureScene():
		flatWaterFog	(0.F, 20.F, D3DCOLOR_XRGB(0, 64, 192)),
		niceWaterFog	(0.F, 20.F, D3DCOLOR_XRGB(0, 192, 255), D3DCOLOR_XRGB(0, 0, 128)),
		demoWaterFog	(0.F, 20.F, D3DCOLOR_XRGB(255, 0, 0), D3DCOLOR_XRGB(0, 255, 0)),
		demoEllipsoid	(D3DXVECTOR3(0.F, 0.F, 0.F), D3DXVECTOR3(5.F, 5.F, 5.F), D3DCOLOR_XRGB(255, 255, 255), 1.F / 5.F),
		demoEllipsoid2	(D3DXVECTOR3(5.F, 0.F, -5.F), D3DXVECTOR3(5.F, 2.F, 5.F), D3DCOLOR_XRGB(0, 0, 0), 1.F / 5.F),
		caveDarkness	(D3DXVECTOR3(10.F, -4.F, -20.F), D3DXVECTOR3(6.F, 2.F, 6.F), D3DCOLOR_XRGB(0, 0, 0), 1.F / 10.F),
		cloudFog		(D3DXVECTOR3(0.F, 5.F, 0.F), D3DXVECTOR3(100.F, 2.F, 100.F), D3DCOLOR_XRGB(255, 255, 0), 1.F / 5.F),
		lastRenderTime	(GetTickCount()),
		panOut2D		(false),
		panIn2D			(false),
		visible2D		(true),
		lastCursorX		(0),
		lastCursorY		(0),
		scroll2D		(0),
		MouseState		(MS_Scroll)
	{
		zFar = 100.F;

		theScene.Add(&the3DScene);
		theScene.Add(&transform2D);

		the3DScene.Add(&background3D);
		the3DScene.Add(&terrain);

		transform2D.SetElement(&the2DScene);

		the2DScene.Add(&theBackground);
		the2DScene.Add(&pageRedir);

		pageRedir.SetElement(pageList[0]);
		curPage = 0;

		distanceFog = 0;
		volFogList	= volumeList;

		scene = &theScene;
	}

	void StartPanIn() {
		if (!panIn2D && (!visible2D || panOut2D)) {
			panOut2D = false;
			panIn2D = true;
			panIn2DStartTime = GetTickCount();
			MouseState = MS_Scroll;
		}
	}

	void StartPanOut() {
		if (!panOut2D && (visible2D || panIn2D)) {
			panIn2D = false;
			panOut2D = true;
			panOut2DStartTime = GetTickCount();
			MouseState = MS_3DCamera;
		}
	}

	void StartPan() {
		if (!panOut2D && (visible2D || panIn2D)) {
			StartPanOut();
		} else {
			StartPanIn();
		}
	}

	virtual void Render(const D3DRenderDevice &device) {
		DWORD time = GetTickCount();

		animTime = (time - lastRenderTime) / 1000.F;
		lastRenderTime = time;

		D3DXMATRIX transf2D;
		D3DXMatrixIdentity(&transf2D);

		enum { panTime = 500 };

		if (panOut2D) {
			DWORD difTime = time - panOut2DStartTime;
			if (difTime > panTime) {
				visible2D = false;
				transform2D.SetElement(0);
				panOut2D = false;
			} else {
				transf2D(3, 0) = (800.F * difTime) / panTime;
				visible2D = true;
				transform2D.SetElement(&the2DScene);
			}
		}

		if (panIn2D) {
			visible2D = true;
			transform2D.SetElement(&the2DScene);

			DWORD difTime = time - panIn2DStartTime;
			if (difTime > panTime) {
				panIn2D = false;
			} else {
				transf2D(3, 0) = 800.F - (800.F * difTime) / panTime;
			}
		}
		transform2D.SetTransform(transf2D);

		D3DXMATRIX pageTransf;
		D3DXMatrixIdentity(&pageTransf);
		pageTransf(3, 1) = float(scroll2D);

		pageRedir.SetTransform(pageTransf);

		Scene::Render(device);
	}

	virtual LRESULT WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool &callDefWndProc) {
		callDefWndProc = true;

		switch (uMsg)
		{
			case WM_KEYDOWN:
				switch (wParam) {
					case VK_F1:
						StartPan();
						return 0;

					case 'Z':
						zFar = 100.F;
						distanceFog = 0;
						break;

					case 'X':
						zFar = fogFar;
						distanceFog = &flatWaterFog;
						break;

					case 'C':
						zFar = fogFar;
						distanceFog = &niceWaterFog;
						break;

					case 'V':
						zFar = fogFar;
						distanceFog = &demoWaterFog;
						break;

					case 'A':
						numVolFog = 0;
						break;

					case 'S':
						volumeList[0] = &demoEllipsoid;
						volumeList[1] = &demoEllipsoid2;
						numVolFog = 2;
						break;

					case 'D':
						volumeList[0] = &caveDarkness;
						numVolFog = 1;
						break;

					case 'F':
						volumeList[0] = &cloudFog;
						numVolFog = 1;
						break;

					case 'Q':
						niceWaterFog.SetBlam(D3DXVECTOR3(0.F, 0.F, 0.F), .0F, D3DXCOLOR(1.F, 1.F, 0.F, 1.F));
						break;

					case 'W':
						niceWaterFog.SetBlam(D3DXVECTOR3(0.F, 0.F, 0.F), .1F, D3DXCOLOR(1.F, 1.F, 0.F, 1.F));
						break;

					case VK_UP:
						scroll2D += 50;
						if (scroll2D > 0) {
							scroll2D = 0;
						}
						break;

					case VK_DOWN:
						scroll2D -= 50;
						if (scroll2D < pageHeightList[curPage]) {
							scroll2D = pageHeightList[curPage];
						}
						break;

					case VK_PRIOR:
						scroll2D += 600;
						if (scroll2D > 0) {
							scroll2D = 0;
						}
						break;

					case VK_NEXT:
						scroll2D -= 600;
						if (scroll2D < pageHeightList[curPage]) {
							scroll2D = pageHeightList[curPage];
						}
						break;

					case VK_LEFT:
						if (curPage > 0) {
							--curPage;
							pageRedir.SetElement(pageList[curPage]);
						}
						break;

					case VK_RIGHT:
						if (curPage < numPages-1) {
							++curPage;
							pageRedir.SetElement(pageList[curPage]);
						}
						break;

					case VK_HOME:
						curPage = 0;
						pageRedir.SetElement(pageList[curPage]);
						break;

					case VK_END:
						curPage = numPages-1;
						pageRedir.SetElement(pageList[curPage]);
						break;
				}
				break;
		}

		bool lButton = (GetKeyState(VK_LBUTTON) & 0x8000) != 0;
		bool rButton = (GetKeyState(VK_RBUTTON) & 0x8000) != 0;
		POINT cursor;
		GetCursorPos(&cursor);
		ScreenToClient(hwnd, &cursor);
		RECT cRect;
		GetClientRect(hwnd, &cRect);
		int deltaX = cursor.x - lastCursorX;
		int deltaY = cursor.y - lastCursorY;

		lastCursorX = cursor.x;
		lastCursorY = cursor.y;

		switch (MouseState) {
			case MS_Scroll:
				if (lButton && cRect.bottom > 0) {
					scroll2D += deltaY * 600 / cRect.bottom;
				}
				switch (uMsg) {
					case WM_RBUTTONDOWN:
						scroll2D = 0;
						return 0;
				}
				if (scroll2D > 0) {
					scroll2D = 0;
				}
				if (scroll2D < pageHeightList[curPage]) {
					scroll2D = pageHeightList[curPage];
				}
				break;

			case MS_3DCamera:
				if (lButton) {
					if (rButton) {
						D3DXVECTOR3 displx(cameraPos.z-cameraLookAt.z, 0.F, cameraLookAt.x-cameraPos.x);
						D3DXVec3Normalize(&displx, &displx);
						displx *= deltaX / 50.F;
						D3DXVECTOR3 displ(cameraLookAt.x-cameraPos.x, 0.F, cameraLookAt.z-cameraPos.z);
						D3DXVec3Normalize(&displ, &displ);
						displ *= deltaY / 50.F;
						displ += displx;
						cameraLookAt -= displ;
						cameraPos -= displ;
					} else {
						float angleX =  deltaX / 200.F;
						float angleY = -deltaY / 200.F;
						D3DXMATRIX m, temp;
						D3DXMatrixTranslation(&m, -cameraPos.x, -cameraPos.y, -cameraPos.z);
						D3DXMatrixMultiply(&m, &m, D3DXMatrixRotationAxis(&temp, &D3DXVECTOR3(cameraLookAt.z-cameraPos.z, 0.F, cameraPos.x-cameraLookAt.x), angleY));
						D3DXMatrixMultiply(&m, &m, D3DXMatrixRotationY(&temp, angleX));
						D3DXMatrixMultiply(&m, &m, D3DXMatrixTranslation(&temp, cameraPos.x, cameraPos.y, cameraPos.z));
						D3DXVec3TransformCoord(&cameraLookAt, &cameraLookAt, &m);
					}
				} else if (rButton) {
					D3DXVECTOR3 displ(cameraPos.z-cameraLookAt.z, 0.F, cameraLookAt.x-cameraPos.x);
					D3DXVec3Normalize(&displ, &displ);
					displ *= deltaX / 50.F;
					displ.y	= deltaY / 50.F;
					cameraLookAt -= displ;
					cameraPos -= displ;
				}
		}

		return 0;
	}
};

GDCLectureScene scene;

D3DRenderDevice device;

HRESULT Render()
{
	if (device) {
		scene.Render(device);
		return device.CheckForLostDevice();
	}
	return D3DERR_NOTAVAILABLE;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		case WM_ERASEBKGND:
			return 1;

		case WM_PAINT:
			switch (Render()) {
				case D3D_OK:
					ValidateRect(hwnd, NULL);
					return 0;

				case D3DERR_NOTAVAILABLE:
					PostQuitMessage(-1);
					return DefWindowProc(hwnd, uMsg, wParam, lParam);
			}
			return 0;

		case WM_KEYDOWN:
			switch (wParam) {
				case VK_ESCAPE:
					PostMessage(hwnd, WM_CLOSE, 0, 0);
					return 0;
				case VK_F1:
					scene.StartPan();
					return 0;
			}
			break;

		default:
			break;
    }

	bool callDefWndProc = true;
	LRESULT lr = scene.WndProc(hwnd, uMsg, wParam, lParam, callDefWndProc);
	if (callDefWndProc) {
		LRESULT lr = device.WndProc(hwnd, uMsg, wParam, lParam, callDefWndProc);
		if (callDefWndProc) {
		    return DefWindowProc(hwnd, uMsg, wParam, lParam);
		} else {
			return lr;
		}
	} else {
		return lr;
	}
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
    static char *APP_NAME = "GDC Lecture";

    // Register the window class for the main window.
    WNDCLASS wc;
    wc.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = (HCURSOR) LoadCursor(hInstance, MAKEINTRESOURCE(IDC_POINTER_BIG));
    wc.hbrBackground = (HBRUSH)NULL;
    wc.lpszMenuName =  NULL;
    wc.lpszClassName = APP_NAME;

    if (!RegisterClass(&wc))
    {
        return -1;
    }

    // Create the main window.
    device.hwnd = CreateWindow(APP_NAME,
                          APP_NAME,
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT,
                          CW_USEDEFAULT,
                          800,
                          600,
                          GetDesktopWindow(),
                          NULL,
                          hInstance,
                          NULL);

    // If the main window cannot be created, terminate
    // the application.
    if (device.hwnd == 0)
    {
        return -1;
    }

    HDC hdc = GetDC(device);
    if(hdc == NULL)
    {
        return -1;
    }

    memset(&device.d3dpp, 0, sizeof(D3DPRESENT_PARAMETERS));
	device.d3dpp.Windowed = TRUE;
	device.d3dpp.hDeviceWindow = device;
	device.d3dpp.BackBufferWidth = 800;
	device.d3dpp.BackBufferHeight = 600;
	device.d3dpp.BackBufferFormat = GetDeviceCaps(hdc, BITSPIXEL) == 16 ? D3DFMT_R5G6B5 : D3DFMT_X8R8G8B8;
	device.d3dpp.BackBufferCount = 1;
	device.d3dpp.EnableAutoDepthStencil = TRUE;
	device.d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	device.d3dpp.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC;

    ReleaseDC(device, hdc);

    HRESULT hr = device.Create();

    if (FAILED(hr)) {
        return -1;
    }

	device.AddGeometry("AquaConv.out");

    ShowWindow(device, SW_SHOWDEFAULT);
    UpdateWindow(device);

    // Now we're ready to recieve and process Windows messages.
    int nret = 1;
    do {
		// Next frame;

		Sleep(10);
		if (Render() == D3DERR_NOTAVAILABLE) {
			nret = 0;
			OutputDebugString("Render disappeared\n");
		}

		{
			MSG msg;
			while ((nret > 0) && PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
				nret = GetMessage(&msg, NULL, 0, 0);
				if (msg.message == WM_QUIT) {
					OutputDebugString("WM_QUIT\n");
				}
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
    } while (nret > 0);

	OutputDebugString("Finalizing...\n");

	if (device) {
	    device.Release();
	}

    DestroyWindow(device);

	OutputDebugString("Finalized...\n");

    return 0;
}

