
#include "StdAfx.h"

#include "GeometryData.h"
#include "D3DDevice.h"

void GeometryData::Render(const D3DRenderDevice &device, const Scene &scene)
{
	if (!m_VB || !m_IB) {
		UnloadData();
		ReloadData(device);
		if (!m_VB || !m_IB) {
			return;
		}
	}

	device->SetVertexShader		(m_FVF);
	device->SetStreamSource		(0, m_VB, m_VertexSize);
	device->SetIndices			(m_IB, 0);
	device->DrawIndexedPrimitive(m_PrimitiveType, 0, m_MaxVertices, 0, m_MaxPrimitives);
}

void GeometryData::CreateBuffers(const D3DRenderDevice &device)
{
	if (!m_VB) {
		device->CreateVertexBuffer(m_MaxVertices * m_VertexSize, D3DUSAGE_WRITEONLY | (m_Dynamic? D3DUSAGE_DYNAMIC: 0), m_FVF, D3DPOOL_DEFAULT, &m_VB);
	}
	if (!m_IB) {
		device->CreateIndexBuffer(m_MaxIndices * sizeof(WORD), D3DUSAGE_WRITEONLY | (m_Dynamic? D3DUSAGE_DYNAMIC: 0), D3DFMT_INDEX16, D3DPOOL_DEFAULT, &m_IB);
	}
}

void GeometryData::DestroyBuffers()
{
	if (m_VB) {
		m_VB->Release();
		m_VB = 0;
	}
	if (m_IB) {
		m_IB->Release();
		m_IB = 0;
	}
}

bool GeometryData::OpenVertexBuffer(const D3DRenderDevice &device, BYTE *&vData, unsigned int num)
{
	CreateBuffers(device);

	if (!m_VB) {
		return false;
	}

	if (num == (unsigned int) -1) {
		num = m_MaxVertices;
	}

	if (m_Dynamic) {
		m_DynLockedVerts = 0;

		if (m_DynVertPos + num > m_MaxVertices) {
			if (FAILED(m_VB->Lock(0, num * m_VertexSize, &vData, D3DLOCK_DISCARD))) {
				return false;
			}
			m_DynVertPos = 0;
		} else {
			if (FAILED(m_VB->Lock(m_DynVertPos * m_VertexSize, num * m_VertexSize, &vData, D3DLOCK_NOOVERWRITE))) {
				return false;
			}
		}

		m_DynLockedVerts = num;
	} else {
		if (FAILED(m_VB->Lock(0, num * m_VertexSize, &vData, 0))) {
			return false;
		}
	}

	return true;
}

void GeometryData::CloseVertexBuffer(unsigned int num)
{
	if (!m_VB) {
		return;
	}
	m_VB->Unlock();

	if (m_Dynamic) {
		if (num == (unsigned int) -1) {
			num = m_DynLockedVerts;
		}
		m_DynVertPos += num;
	}
}

bool GeometryData::OpenIndexBuffer(const D3DRenderDevice &device, WORD *&iData, unsigned int num)
{
	CreateBuffers(device);

	if (!m_IB) {
		return false;
	}

	if (num == (unsigned int) -1) {
		num = m_MaxIndices;
	}

	if (m_Dynamic) {
		m_DynLockedPrims = 0;

		if (m_DynPrimPos + num > m_MaxIndices) {
			if (FAILED(m_IB->Lock(0, num * sizeof(WORD), (BYTE **) &iData, D3DLOCK_DISCARD))) {
				return false;
			}
			m_DynPrimPos = 0;
		} else {
			if (FAILED(m_IB->Lock(m_DynPrimPos * sizeof(WORD), num * sizeof(WORD), (BYTE **) &iData, D3DLOCK_NOOVERWRITE))) {
				return false;
			}
		}

		m_DynLockedPrims = num;
	} else {
		if (FAILED(m_IB->Lock(0, num * sizeof(WORD), (BYTE **) &iData, 0))) {
			return false;
		}
	}

	return true;
}

void GeometryData::CloseIndexBuffer(unsigned int num)
{
	if (!m_IB) {
		return;
	}
	m_IB->Unlock();

	if (m_Dynamic) {
		if (num == (unsigned int) -1) {
			num = m_DynLockedPrims;
		}
		m_DynPrimPos += num;
	}
}
