
#ifndef GeometryData_H_INCLUDED
#define GeometryData_H_INCLUDED

#pragma once

#include <d3d8.h>
#include <d3dx8.h>

class D3DRenderDevice;
class Scene;

class GeometryData {

public:
	DWORD					m_FVF;
	DWORD					m_VertexSize;
	DWORD					m_MaxVertices;
	IDirect3DVertexBuffer8 *m_VB;
	DWORD					m_DynVertPos;
	DWORD					m_DynLockedVerts;

	D3DPRIMITIVETYPE		m_PrimitiveType;
	DWORD					m_MaxIndices;
	DWORD					m_MaxPrimitives;
	IDirect3DIndexBuffer8 * m_IB;
	DWORD					m_DynPrimPos;
	DWORD					m_DynLockedPrims;

	bool					m_Dynamic;

	GeometryData(DWORD fvf, D3DPRIMITIVETYPE primitiveType, bool dynamic = false):
		m_FVF			(fvf),
		m_VertexSize	(D3DXGetFVFVertexSize(fvf)),
		m_MaxVertices	(0),
		m_VB			(0),
		m_DynVertPos	(0),
		m_DynLockedVerts(0),
		m_PrimitiveType	(primitiveType),
		m_MaxIndices	(0),
		m_MaxPrimitives	(0),
		m_IB			(0),
		m_DynPrimPos	(0),
		m_DynLockedPrims(0),
		m_Dynamic		(dynamic)
	{
	}

	virtual ~GeometryData() {
		UnloadData();
	}

	virtual void ReloadData(const D3DRenderDevice &device) {
	}
	void UnloadData() {
		DestroyBuffers();
	}

	virtual void Render(const D3DRenderDevice &device, const Scene &scene);

protected:
	void CreateBuffers(const D3DRenderDevice &device);
	void DestroyBuffers();

	template < class T >
	bool OpenVertexBuffer(const D3DRenderDevice &device, T *&vData, unsigned int num = -1) {
		return OpenVertexBuffer(device, *(BYTE **)&vData, num);
	}

	bool OpenVertexBuffer(const D3DRenderDevice &device, BYTE *&vData, unsigned int num);
	void CloseVertexBuffer(unsigned int num = -1);

	bool OpenIndexBuffer(const D3DRenderDevice &device, WORD *&iData, unsigned int num = -1);
	void CloseIndexBuffer(unsigned int num = -1);

	template < class T >
	bool OpenBuffers(const D3DRenderDevice &device, T *&vData, WORD *&iData, unsigned int numVerts = -1, unsigned int numIdx = -1) {
		if (OpenVertexBuffer(device, vData, numVerts)) {
			if (OpenIndexBuffer(device, iData, numIdx)) {
				return true;
			}
			CloseVertexBuffer(0);
		}

		return false;
	}

	void CloseBuffers(unsigned int numVerts = -1, unsigned int numIdx = -1) {
		CloseIndexBuffer(numIdx);
		CloseVertexBuffer(numVerts);
	}

};

#endif
