
#include "StdAfx.h"

#include "Scene.h"
#include "RenderableElement.h"

void Scene::Render(const D3DRenderDevice &device)
{	
	D3DXCreateMatrixStack(0, &matrixStack);

    if (!FAILED(device->BeginScene())) {
		device->Clear(0, NULL,
			(clear? D3DCLEAR_TARGET: 0) |
			(clearZBuf? D3DCLEAR_ZBUFFER: 0) |
			(clearStencil? D3DCLEAR_STENCIL: 0),
			clearColor,
			1.0f,
			0);

		{
			float screenRatio = GetSystemMetrics(SM_CYSCREEN) / (float) GetSystemMetrics(SM_CXSCREEN);
			RECT crect;
			GetClientRect(device, &crect);

			float windowRatio = crect.bottom / (float) crect.right;

			float ratio = screenRatio * (4.F / 3.F) / windowRatio;

			D3DXMatrixPerspectiveFovLH(&projectionMatrix, fov, ratio, zNear, zFar);
			device->SetTransform(D3DTS_PROJECTION, &projectionMatrix);

			D3DXMatrixInverse(&projectionMatrixInverse, 0, &projectionMatrix);
		}

		{
			D3DXVECTOR3 up	(0.F,  1.F,   0.F);
			D3DXMatrixLookAtLH(&viewMatrix, &cameraPos, &cameraLookAt, &up);
			device->SetTransform(D3DTS_VIEW, &viewMatrix);

			D3DXMatrixInverse(&viewMatrixInverse, 0, &viewMatrix);
			D3DXMatrixMultiply(&viewprojMatrix, &viewMatrix, &projectionMatrix);
			D3DXMatrixInverse(&viewprojMatrixInverse, 0, &viewprojMatrix);

			float frFar = zFar * 1.05F;	// Just a little bit extra.

			D3DXVECTOR4 v0(-frFar, -frFar, frFar, frFar);
			D3DXVECTOR4 v1( frFar, -frFar, frFar, frFar);
			D3DXVECTOR4 v2(-frFar,  frFar, frFar, frFar);
			D3DXVECTOR4 v3( frFar,  frFar, frFar, frFar);

			D3DXVec4Transform(&v0, &v0, &viewprojMatrixInverse);
			D3DXVec4Transform(&v1, &v1, &viewprojMatrixInverse);
			D3DXVec4Transform(&v2, &v2, &viewprojMatrixInverse);
			D3DXVec4Transform(&v3, &v3, &viewprojMatrixInverse);

			frustumMinVals =
			frustumMaxVals = cameraPos;

			if (frustumMinVals.x > v0.x) frustumMinVals.x = v0.x;
			if (frustumMinVals.x > v1.x) frustumMinVals.x = v1.x;
			if (frustumMinVals.x > v2.x) frustumMinVals.x = v2.x;
			if (frustumMinVals.x > v3.x) frustumMinVals.x = v3.x;
			if (frustumMinVals.y > v0.y) frustumMinVals.y = v0.y;
			if (frustumMinVals.y > v1.y) frustumMinVals.y = v1.y;
			if (frustumMinVals.y > v2.y) frustumMinVals.y = v2.y;
			if (frustumMinVals.y > v3.y) frustumMinVals.y = v3.y;
			if (frustumMinVals.z > v0.z) frustumMinVals.z = v0.z;
			if (frustumMinVals.z > v1.z) frustumMinVals.z = v1.z;
			if (frustumMinVals.z > v2.z) frustumMinVals.z = v2.z;
			if (frustumMinVals.z > v3.z) frustumMinVals.z = v3.z;

			if (frustumMaxVals.x < v0.x) frustumMaxVals.x = v0.x;
			if (frustumMaxVals.x < v1.x) frustumMaxVals.x = v1.x;
			if (frustumMaxVals.x < v2.x) frustumMaxVals.x = v2.x;
			if (frustumMaxVals.x < v3.x) frustumMaxVals.x = v3.x;
			if (frustumMaxVals.y < v0.y) frustumMaxVals.y = v0.y;
			if (frustumMaxVals.y < v1.y) frustumMaxVals.y = v1.y;
			if (frustumMaxVals.y < v2.y) frustumMaxVals.y = v2.y;
			if (frustumMaxVals.y < v3.y) frustumMaxVals.y = v3.y;
			if (frustumMaxVals.z < v0.z) frustumMaxVals.z = v0.z;
			if (frustumMaxVals.z < v1.z) frustumMaxVals.z = v1.z;
			if (frustumMaxVals.z < v2.z) frustumMaxVals.z = v2.z;
			if (frustumMaxVals.z < v3.z) frustumMaxVals.z = v3.z;
		}

		if (scene) {
			scene->Render(device, *this);
		}

	/*
		{
			static float a0, a1, a2;

			D3DXMatrixRotationYawPitchRoll(&Cube.m_Transform.m_Transf, a0, a1, a2);

			a0 += .01F;
			a1 += .011F;
			a2 += .00376F;

			Cube.Render(device, *this);
		}
		{
			static float a0, a1, a2;

			D3DXMatrixRotationYawPitchRoll(&Tetrahedron.m_Transform.m_Transf, a0, a1, a2);

			a0 += .01F;
			a1 += .011F;
			a2 += .00376F;

			Tetrahedron.Render(device, *this);
		}
		{
			static float a0, a1, a2;

			D3DXMatrixRotationYawPitchRoll(&File.m_Transform.m_Transf, a0, a1, a2);

			a0 += .01F;
	//			a1 += .011F;
	//			a2 += .00376F;

	//			File.Render(device, *this);
		}

	//		Background.Render(device, *this);

		TextPage1.Render(device, *this);
	*/

		device->EndScene();
	}

	matrixStack->Release();
	matrixStack = 0;
}

