
#ifndef VolumetricFog_H_INCLUDED
#define VolumetricFog_H_INCLUDED

#pragma once

#include <d3d8.h>
#include <d3dx8.h>

struct FogChangeValue {
	float t;
	D3DXCOLOR *color;
};

class FogVolumeDistance {
public:
	virtual ~FogVolumeDistance() {}
	virtual void GetFogValues(
		FogChangeValue *&	changeValueBuffer,
		D3DXCOLOR &			initialColor,
		const D3DXVECTOR3 &	cameraPos,
		D3DXVECTOR3 &		vertexPos,
		float &				distance) = 0;
};

class FogVolumeDistanceFlat: public FogVolumeDistance {
public:
	float m_MinDist;
	float m_MaxDist;
	D3DXCOLOR m_Color;

	float m_MinT;

	FogVolumeDistanceFlat(
		float			minDist,
		float			maxDist,
		const D3DCOLOR &color);

	void GetFogValues(
		FogChangeValue *&	changeValueBuffer,
		D3DXCOLOR &			initialColor,
		const D3DXVECTOR3 &	cameraPos,
		D3DXVECTOR3 &		vertexPos,
		float &				distance);
	virtual void CalculateValues(const D3DXVECTOR3 &cameraPos, const D3DXVECTOR3 &vertexPos, float &minDist, float &maxDist, D3DXCOLOR &color) {}
};

class FogVolumeDistanceVGrad: public FogVolumeDistanceFlat {
public:
	D3DXCOLOR m_HiColor;
	D3DXCOLOR m_LoColor;

	FogVolumeDistanceVGrad(
		float			minDist,
		float			maxDist,
		const D3DCOLOR &hicolor,
		const D3DCOLOR &locolor);

	virtual void CalculateValues(const D3DXVECTOR3 &cameraPos, const D3DXVECTOR3 &vertexPos, float &minDist, float &maxDist, D3DXCOLOR &color);
};

class FogVolumeDistanceHGrad: public FogVolumeDistanceFlat {
public:
	D3DXCOLOR m_HiColor;
	D3DXCOLOR m_LoColor;

	FogVolumeDistanceHGrad(
		float			minDist,
		float			maxDist,
		const D3DCOLOR &hicolor,
		const D3DCOLOR &locolor);

	virtual void CalculateValues(const D3DXVECTOR3 &cameraPos, const D3DXVECTOR3 &vertexPos, float &minDist, float &maxDist, D3DXCOLOR &color);
};

class FogVolumeDistanceVGradBlam: public FogVolumeDistanceVGrad {
public:
	D3DXVECTOR3	m_BlamPos;
	float		m_BlamWidth;
	D3DXCOLOR	m_BlamColor;

	void SetBlam(const D3DXVECTOR3 &blamPos, float blamWidth, const D3DXCOLOR &blamColor) {
		m_BlamPos	= blamPos;
		m_BlamWidth	= blamWidth;
		m_BlamColor	= blamColor;
	}

	FogVolumeDistanceVGradBlam(
		float			minDist,
		float			maxDist,
		const D3DCOLOR &hicolor,
		const D3DCOLOR &locolor);

	virtual void CalculateValues(const D3DXVECTOR3 &cameraPos, const D3DXVECTOR3 &vertexPos, float &minDist, float &maxDist, D3DXCOLOR &color);
};

class FogVolume {
public:
	virtual ~FogVolume() {}
	virtual void GetFogValues(
		FogChangeValue *&	changeValueBuffer,
		D3DXCOLOR &			initialColor,
		const D3DXVECTOR3 &	cameraPos,
		const D3DXVECTOR3 &	vertexPos,
		float				distance) = 0;
};

class FogEllipsoid: public FogVolume {
public:
	D3DXVECTOR3 m_Pos;
	D3DXVECTOR3 m_Scale;

	D3DXCOLOR m_Color;
	D3DXCOLOR m_NegColor;

	FogEllipsoid(
		const D3DXVECTOR3 &	pos,
		const D3DXVECTOR3 &	radii,
		const D3DCOLOR &	color,
		float				density);

	virtual void GetFogValues(
		FogChangeValue *&	changeValueBuffer,
		D3DXCOLOR &			initialColor,
		const D3DXVECTOR3 &	cameraPos,
		const D3DXVECTOR3 &	vertexPos,
		float				distance);
};

D3DXCOLOR ApplyFogsToVertex(
	FogVolumeDistance *	distanceFog,
	FogVolume **		volumeList,
	unsigned int		numVolumes,
	const D3DXVECTOR3 &	vertexPos,
	const D3DXVECTOR3 &	cameraPos);

#endif
